function result = chgofvar(cmd, arg)
% CHANGE OF COORDINATES
%
% This module is for graphing and investigating a change of coordinates.
%
% Enter the domain of u and v values in the top left of the window.  The
% rectangle directly underneath represents the domain.  The right rectangle
% contains the range.  Grey and blue curves here are images of the grey
% and blue curves in the left rangle.  Below, specify a transformation from
% the uv-plane to the xy-plane.  You can also specify a parametric curve.
% It will be plotted in the uv-planne and its image will be plotted in the
% xy-plane.
% 
% To place a colored marker on the domain, click one of the marker buttons
% in the lower right of the window.  When the cursor changes to a crosshair,
% click where you would like the marker to go.  Its image will be drawn in
% the range.  You can also draw six "x"-shaped markers.
%
% 3.0/18-July-1998

% HISTORY:
% 1.0 -- Original by Alex Gottlieb (Summer 1996)
% 2.0 -- Major changes by Tom Insel in consultation with William Stein 
%     (January 1998), including:
%   - changed from a script to a function, in the suggested ``switchyard''
%     format to avoid long callbacks.
%   - used Tags and UserData properties to avoid globals & allow recreation of
%     closed windows.
%   - control window layout improved.
%   - fixed zooming bug & axis labels.
%   - error detection in user input, use cleanup() to catch invisible characters.
%   - made mostly `doubleclick-proof' by using parent arguments for objects.
% 3.0 -- Changed again in June 1998 for MATLAB 5.2
%   - peristent variables
%   - turned into one window, which works at 640x480 or 832x624
%
% REQUIRES: errmesg.m, cleanup.m
%
% TODO/BUGS:
%  - if we try to draw the grid lines in '.' or ':' or '--' style, they don't
%    draw properly until the window is hidden, then shown (this problem seems
%    specific to 4.2c.1 on the 7200/120, but not on my PowerBook 5300cs).

persistent x_of_uv y_of_uv lo_u lo_v hi_u hi_v u_of_t v_of_t lo_t hi_t
persistent red_dot_uv white_dot_uv blue_dot_uv purple_dot_uv
persistent red_dot_xy white_dot_xy blue_dot_xy purple_dot_xy
persistent domain_axes range_axes

% colors

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];
math_hgrid = math_bkgnd;
math_vgrid = [.75 .75 .75];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	chgofvar('regraph');

	% Bring the control figure to the front.

	control_figures = findobj('tag', 'chgofvar_control');
	if length(control_figures) == 0
		control_figures = chgofvar('makecontrol');
	end

	figure(control_figures(1));

elseif strcmp(cmd, 'regraph')

	control_figures = findobj('tag', 'chgofvar_control');
	if length(control_figures) == 0
		control_figures = chgofvar('makecontrol');
	end

	% -- THE DOMAIN

	try
		u0 = str2num(cleanup(get(lo_u, 'string')));
		u1 = str2num(cleanup(get(hi_u, 'string')));
		v0 = str2num(cleanup(get(lo_v, 'string')));
		v1 = str2num(cleanup(get(hi_v, 'string')));
	catch
		errmesg('uv-domain', 'num')
		return
	end

	% The original version used cla here and created the axes only once,
	% when the figure was created, but after resetting the domain
	% twice, zooming always used the second domain. This was probably
	% due to a bug in MATLAB, but to avoid trouble, we delete and
	% recreate the axis here. (bug still present in MATLAB 5.2)
	
	delete(domain_axes)
	domain_axes = axes('position', [0.05 .32 .4 .58]);
	axis square
	
	red_dot_uv = 0;
	white_dot_uv = 0;
	blue_dot_uv = 0;
	purple_dot_uv = 0;

%	axis([u0 u1 v0 v1]);
	
	zoom on
	grid off
	hold on
	xlabel('u'), ylabel('v')
	
	% GRID IN DOMAIN

	u = linspace(u0,u1,200);
	for i = 0:12

		% Here and elsewhere, we extend a scalar to a vector, using 
		% 1 + [0 0 ... 0] = [1 1 ... 1].

		v = v0 + i*(v1 - v0)/12 + 0*u;
		plot(u, v, 'Color', math_hgrid);
	end

	v = linspace(v0,v1,200); 
	for i = 0:12
		u = u0 + i*(u1 - u0)/12 + 0*v;
		plot(u, v, 'Color', math_vgrid)
	end

	% CURVE IN THE DOMAIN

	try
		t0 = str2num(cleanup(get(lo_t, 'string')));
		t1 = str2num(cleanup(get(hi_t, 'string')));
	catch
		errmesg('t-domain', 'num')
		return
	end

	t = linspace(t0, t1, 300);
	
	try
		u_curve = eval(vectorize(cleanup(get(u_of_t, 'string')))) + 0*t;
	catch
		errmesg('u(t)', 'fun');
		return
	end

	try
		v_curve = eval(vectorize(cleanup(get(v_of_t, 'string')))) + 0*t;
	catch
		errmesg('v(t)', 'fun');
		return
	end

	plot(u_curve, v_curve);

	hold off

	% -- THE RANGE

	delete(range_axes)
	range_axes = axes('position', [0.55 .32 .4 .58]);
	axis square

	red_dot_xy = 0;
	white_dot_xy = 0;
	blue_dot_xy = 0;
	purple_dot_xy = 0;

	zoom on
	hold on

	x_string = vectorize(cleanup(get(x_of_uv, 'string')));
	y_string = vectorize(cleanup(get(y_of_uv, 'string')));

	% dummy eval of x_string & y_string to catch errors
	% before the loop.  This is probably bad form, but neater code.	
	
	u = -666;
	v = -666;

	try
		eval([x_string ';']);
	catch
		errmesg('x(u,v)', 'fun')
		return
	end
	
	try
		eval([x_string ';']);
	catch
		errmesg('x(u,v)', 'fun')
		return
	end

	% GRID IN RANGE

	u = linspace(u0,u1,200); 
	for i = 0:12
		v = v0 + i*(v1 - v0)/12 + 0*u;
		x = eval(x_string);
		if length(x) == 1, x = x + 0*u; end,
		y = eval(y_string);
		if length(y) == 1, y = y + 0*u; end,
		plot(x, y, 'Color', math_hgrid);
	end

	v = linspace(v0,v1,200); 
	for i = 0:12
		u = u0 + i*(u1 - u0)/12 + 0*v;
		x = eval(x_string);
		if length(x) == 1, x = x + 0*v; end,
		y = eval(y_string);
		if length(y) == 1, y = y + 0*v; end,
		plot(x, y, 'Color', math_vgrid);
	end

	% CURVE IN RANGE

	% We have already checked x_string and y_string for
	% errors, so we don't do it again.  We do need to eval()
	% again, since u and v have changed.

	u = u_curve;
	v = v_curve;

	x = eval(x_string);
	if length(x) == 1, x = x + 0*u_curve; end
	y = eval(y_string);
	if length(y) == 1, y = y + 0*u_curve; end

	plot(x, y)

	xlabel('x(u,v)')
	ylabel('y(u,v)')

	hold off

elseif strcmp(cmd, 'marker')

	control_figures = findobj('tag', 'chgofvar_control');
	if length(control_figures) == 0
		control_figures = chgofvar('makecontrol');
	end

	axes(domain_axes)

	hold on
	
	if arg == '6'
		[u v] = ginput(6);
	else
		[u v] = ginput(1);
	end

	if arg == '6'
		plot(u, v, 'gx', 'markersize', 8);
	elseif arg == 'r'
		if red_dot_uv, delete(red_dot_uv), end
		red_dot_uv = plot(u, v, 'r.', 'markersize', 20);
	elseif arg == 'g'
		if white_dot_uv, delete(white_dot_uv), end
		white_dot_uv = plot(u, v, 'g.', 'markersize', 20);
	elseif arg == 'b'
		if blue_dot_uv, delete(blue_dot_uv), end
		blue_dot_uv = plot(u, v, 'b.', 'markersize', 20);
	elseif arg == 'm'
		if purple_dot_uv, delete(purple_dot_uv), end
		purple_dot_uv = plot(u, v, 'm.', 'markersize', 20);
	end

	hold off

	try
		x = eval(vectorize(cleanup(get(x_of_uv, 'string'))));
	catch
		errmesg('x(u,v)', 'fun')
		return
	end

	try
		y = eval(vectorize(cleanup(get(y_of_uv, 'string'))));
	catch
		errmesg('y(u,v)', 'fun')
		return
	end

	axes(range_axes)
	
	hold on

	if (arg == '6')	% multiple markers		
		plot( x, y, 'gx', 'markersize', 8);
	elseif arg == 'r'
		if red_dot_xy, delete(red_dot_xy), end
		red_dot_xy = plot( x, y, 'r.', 'markersize', 20);
	elseif arg == 'g'
		if white_dot_xy, delete(white_dot_xy), end
		white_dot_xy = plot( x, y, 'g.', 'markersize', 20);
	elseif arg == 'b'
		if blue_dot_xy, delete(blue_dot_xy), end
		blue_dot_xy = plot( x, y, 'b.', 'markersize', 20);
	elseif arg == 'm'
		if purple_dot_xy, delete(purple_dot_xy), end
		purple_dot_xy = plot( x, y, 'm.', 'markersize', 20);
	end
	
	hold off

elseif strcmp(cmd, 'makecontrol')

	control_figure = figure(...
		'position', [10 0 800 600], ...
		'name', 'Change of Coordinates', ...
		'numbertitle', 'off',...
		'resize', 'off',...
		'Color', math_bkgnd,...
		'renderer', 'zbuffer',... % to prevent flickering with markers
		'Tag', 'chgofvar_control');
		
	control_pos = get(control_figure, 'Position');
	% we play some tricks later to make this look good on 640x480 -- 832x624
	% displays without having to do everything in relative coordinates.

	% The domain controls
	
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 control_pos(4)-30 50 20],...
		'string', 'DOMAIN',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	lo_u = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [60 control_pos(4)-30 40 20],...
		'string', '0',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [100 control_pos(4)-30 30 20],...
		'string', ' u ',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
		
	hi_u = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [130 control_pos(4)-30 40 20],...
		'string', '1',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [170 control_pos(4)-30 20 20],...
		'string', '&',...
		'horizontalalignment', 'center',...
		'backgroundcolor', math_bkgnd);	
		
	lo_v = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [190 control_pos(4)-30 40 20],...
		'string', '-pi',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [230 control_pos(4)-30 30 20],...
		'string', ' v ',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
		
	hi_v = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [260 control_pos(4)-30 40 20],...
		'string', 'pi',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);
		
	domain_axes = axes('position', [0.05 .32 .4 .58]);
	axis square

	% range
	
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [control_pos(3)/2+10 control_pos(4)-30 50 20],...
		'string', 'RANGE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	range_axes = axes('position', [0.55 .32 .4 .58]);
	axis square
	
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 100 control_pos(3)-20 1]);

    % curve

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 70 60 20],...
		'string', 'CURVE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'u(t) =',...
		'position', [70 70 40 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	u_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'sin(t)',...
		'position', [115 70 150 20],...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'text', ...
		'string', 'v(t) =',...
		'position', [275 70 40 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
	   
	v_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'cos(t)',...
		'position', [320 70 150 20],...
		'callback', 'chgofvar(''regraph'');',...
	    'backgroundcolor', math_field);

	lo_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [490 70 40 20],...
		'string', '0',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [530 70 30 20],...
	    'string', ' t ', ...
		'horizontalalignment', 'center',...
		'backgroundcolor', math_bkgnd);
		
	hi_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [560 70 40 20],...
		'string', '2*pi',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);

	% change of variables controls

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 45 150 20],...
		'string', 'CHANGE OF COORDINATES',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [175 45 50 20],...
		'string', 'x(u,v) =',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	x_of_uv = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [230 45 150 20],...
		'string', 'u*cos(v)',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [395 45 50 20],...
		'string', 'y(u,v) =',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	y_of_uv = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [450 45 150 20],...
		'string', 'u*sin(v)',...
		'callback', 'chgofvar(''regraph'');',...
		'backgroundcolor', math_field);	

	% button

	uicontrol(control_figure,...
		'string', 'Help',...
		'position', [10 10 80 30],...
		'callback', 'helpwin chgofvar');

	uicontrol(control_figure,...
		'string', 'Quit',...
		'position', [100 10 80 30],...
		'callback', 'chgofvar(''end'');');
		
	% marker buttons
	
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [control_pos(3)-340 10 60 20],...
		'string', 'MARKERS',...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
		
	uicontrol(control_figure,...
		'string', 'Clear',...
		'position', [control_pos(3)-270 10 60 30],...
		'callback', 'chgofvar(''regraph'');');
	
	uicontrol(control_figure,...
		'string', 'R',...
		'backgroundcolor', 'r',...
		'position', [control_pos(3)-200 10 30 30],...
		'callback','chgofvar(''marker'',''r'');');

	uicontrol(control_figure,...
		'string', 'G',...
		'backgroundcolor', 'g',...
		'position', [control_pos(3)-160 10 30 30],...
		'callback','chgofvar(''marker'',''g'');');

	uicontrol(control_figure,...
		'string', 'B',...
		'backgroundcolor', 'b',...
		'position', [control_pos(3)-120 10 30 30],...
		'callback','chgofvar(''marker'',''b'');');

	uicontrol(control_figure,...
		'string', 'M',...
		'backgroundcolor', 'm',...
		'position', [control_pos(3)-80 10 30 30],...
		'callback','chgofvar(''marker'',''m'');');

	uicontrol(control_figure,...
		'string', '6',...
		'position', [control_pos(3)-40 10 30 30],...
		'callback','chgofvar(''marker'',''6'');');

	result = control_figure;

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'chgofvar_control');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end
