function result = curveanimator(cmd, arg)
% CURVE ANIMATOR
%
% Use this module to visualize the parametrization of a curve
% x(t)*i + y(t)*j.
%
% Input x(t), y(t), and the domain of t values in the appropriate
% boxes.  Adjust the speed.  Press the ANIMATE button to see the 
% curve traced.
%
% 2.1/18-July-1998

% This is basically a front end for MATLAB's comet.m
%
% KNOWN BUGS:
% The speed control works by changing the resolution, which will cause
% problems on fast computers.
%
% REQUIRES: cleanup.m, errmesg.m
%
% HISTORY:
% 1.0 -- Original by Alex Gottlieb (Summer 1996)
% 2.0 -- Rewritten by Tom Insel (February 1998)
%        (eliminated global variables, standardized layout, etc.)
%        bugs fixed: comet trail doesn't immediately disappear.
% 2.1 -- Cleaned up for MATLAB 5, uses only one window.
%        (tinsel, summer 1998)

persistent x_of_t y_of_t lo_t hi_t speed_control

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	control_figures = findobj('tag', 'curveanimator_control');
	if length(control_figures) == 0
		control_figures = curveanimator('makecontrol');
	end

	figure(control_figures(1));

elseif strcmp(cmd, 'regraph')

	control_figures = findobj('tag', 'curveanimator_control');
	if length(control_figures) == 0
		control_figures = curveanimator('makecontrol');
	end
		
	try
		t0 = str2num(cleanup(get(lo_t, 'string')));
		t1 = str2num(cleanup(get(hi_t, 'string')));
	catch
		errmesg('t domain','num');
		return
	end
	
	t = linspace(t0, t1, get(speed_control, 'value'));
	
	try
		x = eval(vectorize(cleanup(get(x_of_t, 'string'))));
	catch
		errmesg('x(t)', 'fun');
		return
	end;

	try
		y = eval(vectorize(cleanup(get(y_of_t, 'string'))));
	catch
		errmesg('y(t)', 'fun');
		return
	end;
	
	comet(x,y);
	
elseif strcmp(cmd, 'makecontrol')

	control_figure = figure(...
		'position', [10 10 600 400], ...
		'name', 'Curve Animator', ...
		'numbertitle', 'off',...
		'resize', 'off',...
		'Color', math_bkgnd,...
		'Tag', 'curveanimator_control');

	% the curve

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 230 100 20],...
		'string', 'CURVE',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'x(t) =',...
		'position', [10 205 50 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);

	x_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'cos(t)',...
		'position', [60 205 130 20],...
		'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'text', ...
		'string', 'y(t) =',...
		'position', [10 180 50 20],...
		'horizontalalignment', 'right',...
		'backgroundcolor', math_bkgnd);
	   
	y_of_t = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'sin(t)',...
		'position', [60 180 130 20],...
	    'backgroundcolor', math_field);

	if strcmp(computer,'MAC2')
		domain_string = ' t ';
	else
		domain_string = '<= t <=';
	end
		
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [80 155 50 20],...
	    'string', domain_string, ...
		'horizontalalignment', 'center',...
		'backgroundcolor', math_bkgnd);

	lo_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [20 155 60 20],...
		'string', '0',...
		'backgroundcolor', math_field);
	   
	hi_t = uicontrol(control_figure,...
		'style', 'edit',...
		'position', [130 155 60 20],...
		'string', '2*pi',...
		'backgroundcolor', math_field);

	% separator

	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 145 180 1]);		

	% the speed slider
	
	uicontrol(control_figure,...
		'style', 'text',...
		'position', [10 115 80 20],...
		'string', 'SPEED',...
		'horizontalalignment', 'left',...
		'backgroundcolor', math_bkgnd);
	
	speed_control = uicontrol(control_figure,...
		'style', 'slider',...
		'position', [50 95 105 16],...
		'min', 100,...
		'max', 1000,...
		'value', 250,...
		'backgroundcolor', math_bkgnd);

	uicontrol(control_figure,...
		'style', 'text',...
		'position', [160 95 30 16],...
		'string', 'slow',...
		'horizontalalignment','left',...
		'backgroundcolor', math_bkgnd);
	
	uicontrol(control_figure,...
		'style', 'text', ...
		'position', [10 95 35 16],...
		'string', 'fast',...
		'horizontalalignment','right',...
		'backgroundcolor', math_bkgnd);

	% separator

	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 85 180 1]);

	% button

	uicontrol(control_figure,...
		'string', 'Animate',...
		'position', [10 45 180 30],...
		'callback', 'curveanimator(''regraph'');');

	uicontrol(control_figure,...
		'string', 'Help',...
		'position', [10 10 85 25],...
		'callback', 'helpwin curveanimator');

	uicontrol(control_figure,...
		'string', 'Quit',...
		'position', [105 10 85 25],...
		'callback', 'curveanimator(''end'');');
		
	axes('position', [.39 .07 .58 .86]);

	result = control_figure;

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'curveanimator_control');
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end

end
