% GAUSSIAN ELIMINATION
%
% This module helps you to practice Gaussian elimination.
% It can also automatically reduce a matrix allowing you
% to step through the row operations performed. 
%
% INSTRUCTIONS
%   1. Select the size of the matrix.  
%   2. Enter the matrix.   
%   3. Using the buttons below the matrix perform various 
%      row operations.  
%      * If at some point you don't know what do, click 
%        on the 'Next' button and the program will 
%        automatically do a reasonable next row operation. 
%      * The 'Do all' button will bring the matrix into
%        reduced row echelon form. 
%   4. Use the 'Undo' and 'Redo' buttons to step through
%      the row operations. 
%   5. If the matrix is square, a bar at the top of the screen 
%      shows the determinant (and if you have the symbolic toolbox)
%      the characteristic polynomial. 
%
% AUTHOR: William Stein, July 1998 

% REQUIRES: cleanup.m, havesymbolic.m 

function Gauss (action, arg)

global h havesym math_bkgnd math_field
math_bkgnd=[.6 .8 1];
math_field = [1 1 1];
 
if nargin == 0
   action = 'init';
   havesym = havesymbolic ;
   m = eye(3) ;
end


switch action
case 'init',
   fig = findobj('tag', 'gauss') ;
   if isempty(fig) 
      fig = figure(...
         'position', [10 20 600 400], ...
         'name', 'GAUSSIAN ELIMINATION', ...
         'numbertitle', 'off',...
         'menu', 'none', ...
         'resize', 'off',...
         'color', math_bkgnd,...
         'Tag', 'gauss') ;
      h = BuildGaussWindow(fig, m) ;
      Gauss update
   else		
      figure(fig(1))
   end
   
case 'changematrix',
   if nargin == 1 
      errordlg ('changematrix: need a matrix to change the matrix.') 
      return 
   end   
   h = BuildGaussWindow(h.fig, arg) ;
   Gauss update
   
case 'entrychanged',
   r = arg(1) ;
   c = arg(2) ;
   try
      h.m(r,c) = tonum(get(h.ed(r,c),'string')) ;	
		addtobuf(['Entry (' num2str(r) ', ' num2str(c) ') changed to ' tostr(h.m(r,c))])       
      Gauss update
   catch
      errordlg (['There is an error in entry (' num2str(r) ',' num2str(c) ').'], ...
         'Input Error') ;
      error(lasterr)
   end
   
case 'examples',
   e=[];
   e(1).button = 'Question 1e' ;
   e(1).text = 'The coefficient matrix for question 1e.' ;
   e(1).ex = [1 2 5 -3 1 10; 2 3 -1 1 -1 2; 1 0 1 0 2 -1] ; 
   e(2).button = 'Problem 1' ;
   e(2).text = 'The coefficient matrix for problem 5.' ;
   e(2).ex = [1 2 2 21; -1 1 -2 -12; 1 2 3 28] ; 
   e(3).button = 'Additional Problem 2' ;
   e(3).text = 'A 5x10 augmented matrix.' ;
   e(3).ex = [0 1 1 1 1    1 0 0 0 0; ...
         1 0 1 1 1    0 1 0 0 0; ...
         1 1 0 1 1    0 0 1 0 0; ...
         1 1 1 0 1    0 0 0 1 0; ...
         1 1 1 1 0    0 0 0 0 1] ; 
   e(4).button = 'Eigenvalue Problem' ;
   e(4).text = 'A 5x5 matrix with integer eigenvalues.' ;
   e(4).ex = [3     0     1     1     0;
     0    -3     0    -3     1;
    -2    -6     0    -6     2;
     0     6     0     6    -2;
     0     0     0     0     4] ;
             
   i = examples('Gauss', e) ;
   if i ~= 0
      Gauss('changematrix',e(i).ex)
   end
   
case 'update',
   % 1. if the corresponding checkboxes are checked, 
	%    display: determinant,
   %             characteristic polynomial
   % 2. make changes in edit fields so that h.lastm = h.m
   if h.r == h.c
      info = ['det(A) = ' tostr(det(h.m))] ;
      if havesym
         info = [info ',  charpoly(A) = ' ...
               tostr(factor(det(sym('t')*sym(eye(h.r))-h.m)))] ;
      end
      set(h.info,'string', info) ;
   end
      
   for r=1:h.r
      for c=1:h.c
         if h.lastm(r,c) ~= h.m(r,c)
            set(h.ed(r,c),'string',tostr(h.m(r,c)))
            h.lastm(r,c) = h.m(r,c) ;
         end
      end
   end
   
   % display last few steps in history box
   s = [] ;
   for i=max(1,h.step-4):min(h.maxstep,h.step+4)
      if i ~= h.step
         t = ['    ' h.buf(i).desc] ;
      else
         t = [' -->' h.buf(i).desc] ;
      end
      s=char(s,t) ;
   end
	set (h.history, 'string', s)       
   
   rowreduce(2)   % highlight pivot
    
case 'swap',
	% SWAP TWO ROWS
	% read row values from popup menus
	row1	=	get(h.swap_first,'value');
	row2	=	get(h.swap_second,'value');

	if row1	~=	row2
		swap (row1, row2)    	
      Gauss update 
   end
   

case 'addmultiple',
	% ADD MULTIPLE OF ONE ROW TO ANOTHER	
	% read row values and the multiplication factor
	row1	=	get(h.add_first,'value') ;
	mult	=	tonum(get(h.add_mult_factor,'string')) ;			
   row2	=	get(h.add_second,'value') ;
   if row1	~=	row2 
	   addmultiple (row1, row2, mult) 
      Gauss update 
   else
      errordlg('Can not add a multiple of a row to itself.') 
   end
   
   
case 'multrow', 
	% MULTIPLY A ROW BY SOME SCALAR
   row =	get(h.mult_row,'value');	% row in which to do multiplication
   mult = tonum(get(h.mult_factor,'string')) ;
   multrow (row, mult) ;
	Gauss update


case 'resize',
	% get the new sizes.
	num_rows = get(h.num_rows,'value') ;
   num_cols = get(h.num_cols,'value') ;
	   
   m = zeros(num_rows,num_cols) ;
   if havesym
      m = sym(m) ;
   end
	for r=1:min(h.r,num_rows)
      for c=1:min(h.c,num_cols)
         m(r,c) = h.m(r,c) ;
		end;
   end;	
   Gauss('changematrix',m)
   
case 'undo',
   if h.step == 1 
      return
   end
	h.step = h.step - 1 ;
	h.m = h.buf(h.step).m ;
   Gauss update
   
case 'redo',
   if h.step < h.maxstep
      h.step = h.step + 1 ;
      h.m = h.buf(h.step).m ;
      Gauss update
   end
   
   
case 'donextstep',
	rowreduce(1)
   
case 'doall',
	rowreduce(0)

   
case 'help',
   helpwin Gauss
   
case 'quit',
   delete (findobj ('tag', 'gauss')) ;
   w = findobj ('tag', 'math.main_menu') ;
   if w
      figure(w(1))
   end
   
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x = a number (either char or symbolic)
% output s = corresponding string representation.
function s = tostr (x)
global havesym
if havesym
   s = char(x) ;
else
   s = num2str(x) ;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% s = string
% output x = a number (either char or symbolic)
function x = tonum (s)
global havesym
s = cleanup(s) ;
if havesym
   x = sym(s) ;
else
   x = str2num(s) ;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% row = row number
% mult = sym or number representing value to multiply by
function multrow (row, mult)
global h 
h.m(row,:) = h.m(row,:) * mult ;
addtobuf(['Multiple row ' num2str(row) ' by ' tostr(mult)]) 
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% row1 = row number
% row2 = row number
% mult = sym or number representing multiple
% function does row2 = mult * row1 + row2 
function addmultiple (row1, row2, mult)
global h 
h.m(row2,:) = h.m(row1,:)*mult + h.m(row2,:) ;
addtobuf(['Add ' tostr(mult) ' * row ' num2str(row1) ' to row ' num2str(row2)]) 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% row1 = row number
% row2 = row number
function swap (row1, row2)
global h
temp = h.m(row1,:) ;
h.m(row1,:) = h.m(row2,:) ;
h.m(row2,:) = temp ;
addtobuf (['Swap row ' num2str(row1) ' and row ' num2str(row2)]) ;


function addtobuf (desc)
global h
h.step = h.step + 1 ;
h.maxstep = h.step ;
h.buf(h.step).m = h.m ;
h.buf(h.step).desc = desc ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% possible actions are:
% 0: do everything
% 1: do a single step
% 2: only highlight the pivot
function rowreduce (action)
global h
% for each column, look for a nonzero entry,
% use it to clear the column, and then swap it to 
% be in the j-th row, where j is the number of 
% pivots found so far.

if action == 2
   set(h.ed,'foregroundcolor','k')
end



j = 0 ;
for c = 1:h.c
   p = 0 ;
   nonzeros = 0 ;
   for r = 1:h.r
      if h.m(r,c) ~= 0 
         if r > j  &  p == 0
            p = r ;
         end
         nonzeros = nonzeros + 1 ; 
      end
   end
   if p ~= 0 & (nonzeros > 1 | h.m(p,c) ~= 1)
      set(h.ed(p,c),'foregroundcolor','b')
      if action == 2
      	return
      end
   end
   if p ~= 0
      j = j + 1 ;
   end
   
   % if there is a nonzero entry in the column,
   % normalize, then use it to clear the column.
   if action ~= 2
      if p ~= 0
         if h.m(p,c) ~= 1
            multrow (p, 1/h.m(p,c)) 
            Gauss update
            if action==1
               return    
            end
         end
         if p ~= j
            swap (p, j) 
            set(h.ed(p,c),'foregroundcolor','k')
            set(h.ed(j,c),'foregroundcolor','b')
            Gauss update
            if action==1
               return
            end
            p = j ;
         end
         
         for r=1:h.r
            if r ~= p & (h.m(r,c) ~= 0)
               addmultiple (p, r, -h.m(r,c))
               Gauss update
               if action==1
                  return    
               end
            end
         end	
      end
   end
end    








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Put the controls for doing row reduction 
% on the matrix m in the window hfig. 
function h = BuildGaussWindow(fig, m)

global havesym math_bkgnd math_field

delete(get(fig,'children')) 

if havesym
   m = sym(m) ;
end
temp = size(m) ; r = temp(1) ; c = temp(2) ;

h.fig = fig ;
h.r = r ;
h.c = c ;
h.step = 1 ;
h.maxstep = 1 ;
h.m = m ;
h.lastm = m ;
h.buf(1).m = m ;
h.buf(1).desc = 'Initial Matrix' ;

% Make det and charpoly info field.														
if r == c
   h.info = uicontrol(fig,'position',[280,370,310,20 ],...
      'backgroundcolor','b',...
      'foregroundcolor','w',...
      'style','edit', ...
      'horizontalalignment', 'left') ;
end

% DRAW THE MATRIX

left=(600-50*c)/2;
top=356-(210-30*r)/2;

for rr=1:r
   for cc=1:c
      h.ed(rr,cc) = uicontrol(fig,...
         'style', 'edit',...
         'position', [50*(cc-1)+left top-30*rr 40 20],...
         'string', tostr(m(rr,cc)),...
         'backgroundcolor', math_field, ...
         'userdata', [rr cc], ...
	      'callback', ...
         'Gauss(''entrychanged'',get(gcbo,''userdata''))');
   end;
end;

% draw brackets
uicontrol(fig,'style','frame',...
   'position', [left-10 top-r*30-10 1 r*30+10]);
uicontrol(fig,'style','frame',...
   'position', [left+50*c top-r*30-10 1 r*30+10]);
uicontrol(fig,'style','frame',...
   'position', [left-10 top-r*30-10 10 1]);
uicontrol(fig,'style','frame',...
   'position', [left-10 top 10 1]);
uicontrol(fig,'style','frame',...
   'position', [left+50*c-10 top-r*30-10 10 1]);
uicontrol(fig,'style','frame',...
   'position', [left+50*c-10 top 10 1]);
uicontrol(fig,'style','text',...
   'position', [left-35 235 20 20],...
   'backgroundcolor',math_bkgnd,...
   'string','A=');


% Create the row reduction control bar

% this string of rows will be used in the below popups.
row_str	=	'Row 1';
for i	=	2:r
   row_str	=	[row_str '|Row ' num2str(i)];
end;

%-----------------------------------------------
% [Add] ___ * ____ into _____
uicontrol(fig,'string', 'Add',...
   'position',[10 91 75 25],...
   'callback','Gauss addmultiple');

% multiplication factor
h.add_mult_factor = uicontrol(fig,'style','edit',...
   'position',[95 92 30 20],...
   'string','1',...
   'backgroundcolor','w');

% * label
uicontrol(fig,'style','text',...
   'position',[130 92 20 15],...
   'backgroundcolor',math_bkgnd,...
   'foregroundcolor','k',...
   'horizontalalignment', 'left', ...
   'string','x');

% row to multiply by
h.add_first=uicontrol(fig,'style','popup',...
   'position',[142 92 70 20],...
   'value',1,...
   'backgroundcolor', math_field,...
   'string',row_str);

% "into" label
uicontrol(fig,'style','text',...
   'position',[215 92 50 15],...
   'backgroundcolor',math_bkgnd,...
   'foregroundcolor','k',...
   'horizontalalignment', 'left', ...
   'string','into');

% row to add into
h.add_second=uicontrol(fig,'style','popup',...
   'position',[240 92 70 20],...
   'value',2,...
   'string',row_str,...
   'backgroundcolor', math_field);

%-----------------------------------------------
% [Swap] ____ with ____
uicontrol(fig,'string', 'Swap',...
   'position',[10 57 75 25],...
   'callback','Gauss swap');

% first row
h.swap_first=uicontrol(fig,'style','popup',...
   'position',[95 60 70 20],...
   'value',1,...
   'string',row_str, ...
   'backgroundcolor', math_field);

% "with" label
uicontrol(fig,'style','text',...
   'position',[170 60 40 15],...
   'backgroundcolor',math_bkgnd,...
   'foregroundcolor','k',...
   'horizontalalignment', 'left', ...
   'string','with');

% second row
h.swap_second=uicontrol(fig,'style','popup',...
   'position',[200 60 70 20],...
   'value',2,...
   'string',row_str, ...
   'backgroundcolor', math_field);	


%-----------------------------------------------

%-----------------------------------------------
% [Mult] ____ by ______

uicontrol(fig,'string', 'Multiply',...
   'position',[10 22 75 25],...
   'callback','Gauss multrow');

% row to multiply by
h.mult_row=uicontrol(fig,'style','popup',...
   'position',[95 27 70 20],...
   'value',1,...
   'string',row_str, ...
   'backgroundcolor', math_field);

% 'by' label
uicontrol(fig,'style','text',...
   'position',[170 27 20 15],...
   'backgroundcolor',math_bkgnd,...
   'foregroundcolor','k',...
   'horizontalalignment', 'left', ...
   'string','by');

% multiplication factor
h.mult_factor = uicontrol(fig,'style','edit',...
   'position',[185 27 30 20],...
   'string','1',...
   'backgroundcolor','w');

%-----------------------------------------------


% make number of rows and number of columns pop ups.
uicontrol(fig,'style','text',...
   'string','ROWS: ',...
   'horizontalalignment','left',...
   'position',[20 370 45 20],...
   'backgroundcolor',math_bkgnd);
h.num_rows=uicontrol(fig,'style','popup',...
   'position',[65 370 50 20],...
   'value',r,...
   'string',['1|2|3|4|5|6'],...
   'backgroundcolor', math_field, ...
   'callback','Gauss resize');
uicontrol(fig,'style','text',...
   'string','COLUMNS:',...
   'horizontalalignment','left',...
   'position',[130 370 75 20],...
   'backgroundcolor',math_bkgnd);
h.num_cols=uicontrol(fig,'style','popup',...
   'position',[195 370 60 20],...
   'value',c,...
   'string',['1|2|3|4|5|6|7|8|9|10'],...
   'backgroundcolor', math_field, ...
   'callback','Gauss resize');



% History list
h.history = uicontrol(fig,'position',[320,45,155,100 ],...
   'backgroundcolor','b',...
   'foregroundcolor','w',...
   'horizontalalignment', 'left', ...
   'style','text') ;
uicontrol (fig, 'position', [320 10 75 30], ...
   'string', 'Undo', ...
   'callback', 'Gauss undo') ;
uicontrol (fig, 'position', [400 10 75 30], ...
   'string', 'Redo', ...
   'callback', 'Gauss redo') ;



uicontrol (fig, ...
   'position', [485 115 55 30],...
   'string', 'Next',...
   'callback', 'Gauss donextstep') ;

uicontrol (fig, ...
   'position', [540 115 55 30],...
   'string', 'Do All',...
   'callback', 'Gauss doall') ;

uicontrol (fig, ...
   'position', [485 80 110 30],...
   'string', 'Examples',...
   'callback', 'Gauss examples') ;

uicontrol (fig, ...
   'position', [485 45 110 30],...
   'string', 'Help',...
   'callback', 'Gauss help') ;

uicontrol (fig, ...
   'position', [485 10 110 30],...
   'string', 'Quit',...
   'callback', 'Gauss quit') ;


