% INTEGRATION TOOL
%
% This module can be used to compute definite or indefinite integrals.
%
% 1. Select either indefinite or definite integration.
% 2. Enter the function to be integrated.
% 3. If the integral is definite, enter the limits of integration.
% 4. Click on the compute button.  The output is not updated 
%    before you do this. 
% 5. The integral will then be computed and displayed.  A graph of
%    f(x) is drawn and, if the integral is indefinite, a graph of an
%    antiderivative of f(x) is also displayed.  In the latter case the 
%    plotted x range can be adjusted.  If the "start at x-axis" 
%    button is checked then the antiderivative is chosen so that
%    it is 0 at x0, by adding a (often hideous) constant. 
% 

function Integrator (action)

persistent h isdefinite

if nargin == 0
   action = 'init' ;
end

switch action
case 'init'
   if ~havesymbolic
	  errordlg('This module requires the symbolic toolbox.', 'Integrator');
	  return;
   end
   hfig = findobj ('tag', 'Integrator') ;
   if isempty (hfig)
      h = CreateIntegratorWin ;
      isdefinite = 1 ;
      Integrator 'update' 
   else
      figure hfig(1) 
   end

case 'definite_radio',
   % user clicked on definite radio button
   set (h.definite_radio, 'value', 1) 
   set (h.indefinite_radio, 'value', 0) 
   set (h.def_stuff, 'visible', 'on') 
   set (h.indef_stuff, 'visible', 'off') 
   if get(h.ax2,'children')
  		delete(get(h.ax2,'children'))
	end
   isdefinite = 1 ;
   Integrator update
   
case 'indefinite_radio',
   % user clicked on indefinite radio button
   set (h.definite_radio, 'value', 0) 
   set (h.indefinite_radio, 'value', 1) 
   set (h.def_stuff, 'visible', 'off') 
   set (h.indef_stuff, 'visible', 'on') 
   axes (h.ax2) 
   grid on
   isdefinite = 0 ;
   Integrator update
   
case 'help',
	helpwin integrator
   
case 'quit',
   delete (findobj ('tag', 'Integrator')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
   
case 'update',
   % user asks to compute the integral and display
   try
      try
         f = sym(cleanup(get(h.f,'string'))) ;
      catch
         errordlg('Something is wrong with the definition of f(x).', 'Input Error')
         error(lasterr)
      end
      if isempty(f)
         errordlg('You must enter a value for f(x).', 'Integrator') ;
         error('f(x) not yet entered.') ;
      end
      
      if isdefinite
         try
            a = sym(cleanup(get(h.a,'string'))) ;
         catch
            errordlg(['Something is wrong with the definition ' ...
                  'of the lower limit of integration.'], 'Input Error')
            error(lasterr)
         end
         if isempty(a)
            errordlg('You must enter a value for the lower limit of integration.', 'Integrator') ;
            error('lower limit not yet entered.') ;
         end
         
         
         try
            b = sym(cleanup(get(h.b,'string'))) ;
         catch
            errordlg(['Something is wrong with the definition ' ...
                  'of the upper limit of integration.'], 'Input Error')
            error(lasterr)
         end
         
         if isempty(b)
            errordlg('You must enter a value for the upper limit of integration.', 'Integrator') ;
            error('upper limit not yet entered.') ;
         end
         
         intf = int (f,a,b) ;
         
         axes(h.ax1)   
         ezplot(f,eval(a),eval(b))
         grid on
         indef = int (f) ;
         set(h.ax2,'visible','off') 
	      if get(h.ax2,'children')
   		   delete(get(h.ax2,'children'))
		   end
         
         set (h.intf, 'string', ['  ' char(intf) ' = ' ...
               num2str(eval(intf))]) 
      
         
      else
         intf = int (f) ;
         try 
            xmin = str2num (get (h.xmin, 'string')) ;
         catch
            errordlg(['Something is wrong with lower x range.'], 'Input Error')
            error(lasterr)
         end
         try 
         	xmax = str2num (get (h.xmax, 'string')) ;
         catch
            errordlg(['Something is wrong with upper x range.'], 'Input Error')
            error(lasterr)
         end
         if (get(h.start_at_x_axis,'value'))
	         x=xmin;		
   	      x0=eval(intf);
            intf=intf-x0;
         end
         
         axes(h.ax1)   
         ezplot(f, xmin, xmax)
         grid on
         xlim = get(h.ax1,'xlim') ;
         axes(h.ax2)
         set(h.ax2,'visible','off') 
         ezplot(intf, xlim(1), xlim(2))
         grid on
         lim1 = min(min(get(h.ax1, 'ylim')),min(get(h.ax2,'ylim'))) ;
         lim2 = max(max(get(h.ax1, 'ylim')),max(get(h.ax2,'ylim'))) ;
         set(h.ax1,'ylim',[lim1 lim2]) ;
         set(h.ax2,'ylim',[lim1 lim2]) ;
         set(get(h.ax2,'children'),'color','r')
	      set (h.intf, 'string', ['  ' char(intf)]) 
         
      end
      
      
   catch
      errordlg('Sorry, I couldn''t compute the integral.',...
            'Graphing Error')
      error(lasterr)
   end   
   
   
end




function h = CreateIntegratorWin()
col = [.6 .8 1] ;
edcol = 'w' ;

h0 = figure('Color',col, ...
	'MenuBar','none', ...
	'Name','INTEGRATOR', ...
	'NumberTitle','off', ...
	'Position',[25   50   610   410], ...
	'Resize','off', ...
   'Tag','Integrator');
h.fig = h0 ;


h.ax1 = axes('Parent',h0, ...
	'Units','pixels', ...
	'Position',[40 170 250 200]);
h.ax2 = axes('Parent',h0, ...
	'Units','pixels', ...
	'Position',[340 170 250 200]);



% ()Indefinite  ()Definite
h.definite_radio = uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'Callback','Integrator ''definite_radio''', ...
	'Position',[20 65 60 20], ...
	'String','Definite', ...
   'Style','radiobutton', ...
   'value', 1);
h.indefinite_radio = uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'Callback','Integrator ''indefinite_radio''', ...
	'Position',[90 65 70 20], ...
	'String','Indefinite', ...
	'Style','radiobutton');


% f(x) = [      ]
uicontrol('Parent',h0, ...
   'BackgroundColor',col, ...
	'HorizontalAlignment','left', ...
	'Position',[20 40 40 20], ...
	'String','f(x) =', ...
	'Style','text');
h.f = uicontrol('Parent',h0, ...
	'BackgroundColor',[1 1 1], ...
   'HorizontalAlignment','left', ...
	'Position',[60 40 150 20], ...
   'Style','edit',...
   'string','sin(x)');


% from a = [ ] to b = [ ]
h.ds1 = uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'HorizontalAlignment','left', ...
	'Position',[20 15 50 20], ...
	'String','From a =', ...
	'Style','text');
h.a = uicontrol('Parent',h0, ...
	'BackgroundColor', 'w', ...
	'Position',[75 15 40 20], ...
   'Style','edit',...
   'string','0');
h.ds2 = h.a ;
h.ds3 = uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'Position',[125 15 40 20], ...
	'String','to b =', ...
	'Style','text');
h.b = uicontrol('Parent',h0, ...
	'BackgroundColor', 'w', ...
	'Position',[170 15 40 20], ...
   'Style','edit',...
   'string','pi/2');
h.ds4 = h.b ;

% x range [ ] [ ]
xrange = uicontrol (h0, 'backgroundcol',col,...
   'horizontalalignment','left',...
   'position',[250 65 80 20],...
   'string','x range:',...
   'style','text',...
   'visible','off') ;
h.xmin = uicontrol (h0, 'backgroundcol','w',...
   'position',[300 65 40 20],...
   'string','-1',...
   'style','edit',...
   'visible','off') ;
h.xmax = uicontrol (h0, 'backgroundcol','w',...
   'position',[350 65 40 20],...
   'string','1',...
   'style','edit',...
   'visible','off') ;

h.start_at_x_axis = uicontrol (h0, 'background', col, ...
   'position', [420 65 120 20],...
   'string', 'Start at X-axis',...
   'style', 'checkbox',...
   'value',0,...
   'visible','off');


% [Update] [Help] [Quit]
uicontrol('Parent',h0, ...
	'Callback','Integrator ''update''', ...
	'Position',[250 15 110 30], ...
	'String','Compute');
uicontrol('Parent',h0, ...
	'Callback','Integrator ''help''', ...
	'Position',[370 15 110 30], ...
	'String','Help');
uicontrol('Parent',h0, ...
	'Callback','Integrator ''quit''', ...
	'Position',[490 15 110 30], ...
	'String','Quit');


uicontrol(h0,'style','frame',...
   'position',[20 95 570 1]) ;


% Int f(x)dx = [        ]
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'HorizontalAlignment','right', ...
	'Position',[100 105 80 20], ...
	'String','Int f(x) dx = ', ...
	'Style','text');
h.intf = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'HorizontalAlignment', 'Left', ...
	'Position',[190 110 400 16], ...
	'Style','text');


h.def_stuff = [h.ds1 h.ds2 h.ds3 h.ds4] ;
h.indef_stuff = [h.xmin h.xmax xrange h.ax2 h.start_at_x_axis] ;
