% LINEARSYSTEM
%
% This module plots a system of up to 3 linear equations.
%
% Instructions:
% 1. Enter the coefficients of the system.  (A blank
%    is considered =0).  
% 2. Each linear equation will be drawn as you enter it.
%    Note that the planes might not look square because
%    they are clipped by the bounding box.
% 3. Rotate your viewpoint by clicking on the graph and draging.
% 4. Click on the zoom buttons to zoom in and out.
%    HINT: If a plane is not visible, zoom out.
% 5. Look in the upper left corner for information about the
%    solution set.   The solution set is also drawn in cyan.
%
% Tip: To visualize 2d graphs rotate the axis so that
%      you are looking down from above.
%      
% William Stein (was@math.berkeley.edu), June 1998.


function LinearSystem (action)

persistent h_fig h_ax h_axbox...
     h_a h_b h_c h_d numeqns h_solution...  % coefficients
     s % zoom factor

if nargin == 0
   action = 'init' ;
end

switch action
   
case 'init',
   h_fig = findobj ('Tag', 'LinearSystem') ;
   if (isempty (h_fig))
      h = CreateLinearSystemWindow ;
      h_fig = h(1) ;
      h_ax = h(2) ;
      h_a(1) = h(3) ;
      h_b(1) = h(4) ;
      h_c(1) = h(5) ;
      h_d(1) = h(6) ;
      h_a(2) = h(7) ;
      h_b(2) = h(8) ;
      h_c(2) = h(9) ;
      h_d(2) = h(10) ;
      h_a(3) = h(11) ;
      h_b(3) = h(12) ;
      h_c(3) = h(13) ;
      h_d(3) = h(14) ;
      h_axbox = h(15) ;
      h_solution = h(16) ;
      numeqns = 3 ;
      s = 20 ;
   end
   figure (h_fig) 
   LinearSystem 'draw'
   
case 'reset',
   set ([h_a h_b h_c h_d], 'string', '') 
   hold off
   delete (get (h_ax, 'children')) 
   linearsystem draw
   
case 'zoomin',
   s = s / 2 ;
   LinearSystem('draw') 
   
case 'zoomout',
   s = s * 2 ;
   LinearSystem('draw') 
   
case 'draw',
   hold off
   set (h_fig, 'renderer', 'zbuffer') ;
   axis ([-s,s,-s,s,-s,s])
   delete (get(h_ax,'children')) ;
   hold on
   col = ['r', 'g', 'b'] ;
   clear m v
   numnonzero = 0 ;
   try
      % draw planes
      for i = 1:numeqns 
         if (empty(h_a(i)))
            a = 0 ;
         else
            a = str2num (cleanup (get(h_a(i), 'string'))) ;
         end
         if (empty(h_b(i)))
            b = 0 ;
         else
            b = str2num (cleanup (get(h_b(i), 'string'))) ;
         end
         if (empty(h_c(i)))
            c = 0 ;
         else
            c = str2num (cleanup (get(h_c(i), 'string'))) ;
         end
         if (empty(h_d(i)))
            d = 0 ;
         else
            d = str2num (cleanup (get(h_d(i), 'string'))) ;
         end
         
         if a 
            fill3([(d-b*s-c*s)/a, (d+b*s-c*s)/a, ...
                  (d+b*s+c*s)/a, (d-b*s+c*s)/a],...
               [s,-s,-s,s],[s,s,-s,-s], col(i));
         elseif b
            fill3([s,-s,-s,s],...
               [(d-a*s-c*s)/b, (d+a*s-c*s)/b, ...
                  (d+a*s+c*s)/b, (d-a*s+c*s)/b],...
               [s,s,-s,-s], col(i)) ;
            
         elseif c
            fill3([s,-s,-s,s],[s,s,-s,-s], ...
               [(d-a*s-b*s)/c, (d+a*s-b*s)/c, ...
                  (d+a*s+b*s)/c, (d-a*s+b*s)/c], col(i)) ;
         elseif d
            delete (get(h_ax,'children'))
            drawnow
            numnonzero=-1;
            return
         end
         if a|b|c
            numnonzero = numnonzero + 1 ;
         end
         m(i,:) = [a b c] ;
         v(i,1) = d ;
      end
      
   catch
      errordlg('There is an error in one of the coefficients.', 'Input Error') 
      error(lasterr) 
   end
   
   % draw solution set
   if numnonzero == -1
	   set(h_solution,'string','There is no solution.')
   end
   
   if numnonzero == 0
      set (h_solution, 'string', ...
         '  The solution set is all of three space') 
   end
   
   if numnonzero == 1
      set(h_solution,'string',...
         '  The solution set is a plane.') 
   end
   if numnonzero > 1
      k = null(m) ;
      if isempty(k)
         p=m^(-1)*v;
         plot3(p(1),p(2),p(3),'c*-','linewidth',10) 
         set(h_solution,'string',...
             char('  The unique solution is:  ', ...
               ['   (' num2str(p(1)) ',' num2str(p(2)) ',' ...
                  num2str(p(3)) ')'])) ;
      else
         if havesymbolic
            w = sym(m)\sym(v) ;
            if w(1) == inf
               set (h_solution, 'string', ...
                  'There is no solution.') 
            else
               
               for i=1:3
		            p1(i)=str2num(char(w(i)-3*s*k(i)));
                  p2(i)=str2num(char(w(i)+3*s*k(i)));
               end
		         set(h_solution,'string',...
      	       char('  The solution is the line  ', ...
                ['   (' char(w(1)) ', ' char(w(2)) ', ' ...
                   char(w(3)) ')  + t( '...
                   num2str(k(1)) ',' num2str(k(2)) ',' ...
                   num2str(k(3)) ')'], ...
                     '  as t varies from -oo to oo')) ;
               
               plot3([p1(1) p2(1)],[p1(2) p2(2)],[p1(3) p2(3)],'c*-',...
                  'linewidth',5)
            end
         else
            set(h_solution,'string','Solution info disabled.')
         end
      end   
   end
   
   
   
   
   
   
   rotate3d on
   
   if (get(h_axbox,'value'))
      axis on
	   axis vis3d
   	grid on
   	box on
     	xlabel('X axis')
   	ylabel('Y axis')
   	zlabel('Z axis')
   else
      axis off
   end
         
case 'help',
   helpwin('linearsystem.m') 

case 'library',
   % bring up a library of interesting examples
   msgbox ('welcome to the unimplemented library...') ;
   %librarydlg ('Linear System Library', 'LinearSystem', ...
   %   {'An example of a 2d system with no solutions.', ...
   %    'A 
   % titles
      % descriptions
   
case 'quit',
   delete (findobj ('tag', 'LinearSystem')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
   
case 'dolibrary',
   % reset 
   switch libarg
   case 1,
      
      
   end
   
   % redraw 
end

function r = empty (h)
s = get(h, 'string') ;
r = 1 ;
for i=1:length(s)
   if s(i) ~= ' '
      r = 0 ;
      return 
   end
end
return 

function r = CreateLinearSystemWindow()
col = [.6 .8 1] ;
eq1col = [1 .7 .7] ;
eq2col = [.7 1 .7] ;
eq3col = [.7 .7 1] ;

h_fig = figure('Color',[0.6 0.8 1], ...
	'MenuBar','none', ...
	'Name','SYSTEM OF LINEAR EQUATIONS', ...
	'NumberTitle','off', ...
	'Position',[10 10 600 420], ...
	'Resize','off', ...
   'Tag','LinearSystem') ;

h_ax = axes('Parent',h_fig, ...
	'Units','pixels', ...
	'CameraUpVector',[0 1 0], ...
	'Position',[260 40 330 370], ...
	'xlim', [-1 1], 'ylim', [-1 1]) ;



h_a1 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq1col, ...
	'Position',[30 290 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[62 290 20 20], ...
	'String','x + ', ...
	'Style','text');
h_b1 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq1col, ...
	'Position',[80 290 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[112 290 20 20], ...
	'String','y + ', ...
   'Style','text');
h_c1 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq1col, ...
	'Position',[130 290 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[162 290 20 20], ...
	'String','z = ', ...
	'Style','text');
h_d1 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq1col, ...
	'Position',[180 290 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')');


h_a2 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq2col, ...
	'Position',[30 260 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[62 260 20 20], ...
	'String','x + ', ...
	'Style','text');
h_b2 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq2col, ...
	'Position',[80 260 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[112 260 20 20], ...
	'String','y + ', ...
   'Style','text');
h_c2 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq2col, ...
	'Position',[130 260 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[162 260 20 20], ...
	'String','z = ', ...
	'Style','text');
h_d2 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq2col, ...
	'Position',[180 260 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')');





h_a3 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq3col, ...
	'Position',[30 230 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[62 230 20 20], ...
	'String','x + ', ...
	'Style','text');
h_b3 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq3col, ...
	'Position',[80 230 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[112 230 20 20], ...
	'String','y + ', ...
   'Style','text');
h_c3 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq3col, ...
	'Position',[130 230 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')') ;
uicontrol('Parent',h_fig, ...
	'BackgroundColor',col, ...
	'Position',[162 230 20 20], ...
	'String','z = ', ...
	'Style','text');
h_d3 = uicontrol('Parent',h_fig, ...
	'BackgroundColor',eq3col, ...
	'Position',[180 230 30 20], ...
   'Style','edit', ...
   'Callback', 'LinearSystem(''draw'')');


h_axbox = uicontrol ('Parent', h_fig, ...
   'Style', 'checkbox', ...
   'Backgroundcolor', col, ...
   'value', 1, ...
   'String', 'Show Axis', ...
   'Position', [85 185 70 20], ...
   'Callback', 'LinearSystem(''draw'')') ;


uicontrol('Parent',h_fig, ...
	'Callback','LinearSystem ''zoomout''', ...
	'Position',[30 140 80 30], ...
   'String','Zoom OUT') ;
uicontrol('Parent',h_fig, ...
	'Callback','LinearSystem ''zoomin''', ...
	'Position',[130 140 80 30], ...
   'String','Zoom IN') ;
uicontrol('Parent',h_fig, ...
	'Callback','LinearSystem ''help''', ...
	'Position',[30 100 80 30], ...
	'String','Help');
%uicontrol('Parent',h_fig, ...
%	'Callback','LinearSystem ''library''', ...
%	'Position',[130 100 80 30], ...
%   'String','Library') ;
uicontrol('Parent',h_fig, ...
	'Callback','LinearSystem ''reset''', ...
	'Position',[30 60 80 30], ...
   'String','Reset') ;
uicontrol('Parent',h_fig, ...
	'Callback','LinearSystem ''quit''', ...
	'Position',[130 60 80 30], ...
	'String','Quit') ;


uicontrol('Parent',h_fig, ...
	'BackgroundColor', col, ...
	'Position',[20 20 200 20], ...
	'String','Click on graph and drag to rotate.', ...
   'Style','text') ;

h_solution=uicontrol(h_fig,...
   'style', 'text', ...
   'backgroundcolor', 'b', ...
   'horizontalalignment', 'left', ...
   'foregroundcolor', 'w', ...
   'position',[20 350 200 50]) ;

r = [h_fig h_ax ...
     h_a1 h_b1 h_c1 h_d1 ...
     h_a2 h_b2 h_c2 h_d2 ...
     h_a3 h_b3 h_c3 h_d3 ...
     h_axbox h_solution] ;