% MassOnASpring
%
% This program graphs and animates solutions to a given second
% order linear differential equation 
%          mu''(t) + cu'(t) + ku(t) = F(t)
% with constant coefficients.    One can view solutions to this 
% type of equation as describing the motion of a mass on a spring.  
%
% INSTRUCTIONS
% 1. Adjust the parameters m, c, k by either typing them in
%    or using the slider bar.
%         m = mass
%         c = damping constant
%         k = spring constant (as defined by Hooke's law).
% 2. Enter the external force F(t). 
% 3. Click on the graph to adjust the starting height. 
% 4. Click on animate to watch two springs bounce up and down.
%    One will move along at constant speed tracing out the 
%    solution curve.
%
% REFERENCES
% To understand the mathematics used in this model please
% read section 3.8-3.9 of "Elementary Differential Equations
% and Boundary Value Problems" by Boyce and Diprima, 6th Ed.
%
% Author: William Stein, July 1998

function dy = MassOnASpring (action, y)

persistent h F m c k maxt miny maxy T Y tsteps u0 uprime0 solution ...
   delay

if nargin == 0 
   fig = findobj ('Tag', 'MassOnASpring') ;
   if (fig)
      figure (fig(1))
   else
      h = CreateMassOnASpringWindow ;
      miny = -5 ;
      maxy = 5 ;
      maxt = 50 ;
      tsteps = 150 ; 
      %delay = 20000 ;
      u0 = -2 ;
      uprime0 = 0 ;
      MassOnASpring update 
   end
   return 
end

if nargin == 2    % we're being called by an ode solver
   t = action ;
   Fval = eval(F) ;
   dy = [y(2) ; (Fval - c* y(2) - k*y(1))/m] ;
   return
end

switch action
case 'update',
   try
      m = eval (cleanup (get(h.m, 'String'))) ;
      if m < .01
         m = .01 ;
         set (h.m,'string','.01') 
         errordlg('Readjusting mass to be .01','Input Error') 
      end
   catch
      errordlg('There is an error in the m input field.','Input Error')
   	error(lasterr)   
   end
   
   try
   	c = eval (cleanup (get(h.c, 'String'))) ;
	catch
      errordlg('There is an error in the c input field.','Input Error')
      error(lasterr)
   end
   
   try
      k = eval (cleanup (get(h.k, 'String'))) ;
   catch
      errordlg('There is an error in the k input field.','Input Error')
      error(lasterr)
   end
   
   try
   	F = cleanup (get (h.F, 'String')) ;
   catch
      errordlg('There is an error in the F input field.','Input Error');
      error(lasterr)
   end
   
   try
      [T,Y] = ode45 ('MassOnASpring', 0:maxt/tsteps:maxt, [u0 uprime0]) ;
   catch
      errordlg(['There was an error solving the differential equation.'...
            '  This is probably the result of an error in the input.'],'Input Error');
      error(lasterr)
   end
   
   set(h.startheight,'string',...
      ['Click mouse to adjust start height.  (' ...
         num2str(u0,'%2.2f') ')']) 
   hold off
   solution=plot(T,Y(:,1)) ;
   hold on
   
   axis off
   line([0 maxt], [0 0], 'color', 'k') 
   line([0 0], [miny maxy], 'color', 'k') 
   text(.2,maxy-.5,num2str(maxy)) 
   text(maxt,0,num2str(maxt))
   axis([-4 maxt miny maxy]) ;
   
case 'animate', 
   set(solution, 'color', [.8 .8 1]) ;
   PointerLoc=get(0,'PointerLocation');
   
	%   block_width = .5 ;
	%   block_height = .5 ;
	block_width = m/30+.5 ;
	block_height = m/30+.5 ;
   block_color = [.8 0 .8] ;
   spring_width = .2 ;
   spring_color = [1 0 0] ;
   spring_numjags = min(4+20*k,40);
   t0=T(1); y0=Y(1,1) ;
   
 %  vel = line([t0],[0],'color','g') ;
 %  set(vel,'erasemode','xor') ;
   
   spring1=line([0], [0], 'color', 'r') ;
  	spring2=line([0], [0], 'color', 'r') ;

   set([spring1 spring2], 'erasemode', 'xor') ;
   jagged = [] ;
   for j=1:spring_numjags
      jagged(j) = (-1)^j * spring_width ;
   end
   jagged(spring_numjags)=0;
	set([spring1 spring2], 'xdata', jagged, ...
		   'ydata', linspace(maxy,y0,spring_numjags+t0)) ;
  	 
   block1=fill(-3/2*block_width+[t0-block_width t0+block_width t0+block_width t0-block_width], ...
      [y0 y0 y0-block_height y0-block_height], [0 1 .5]) ;
   block2=fill([t0-block_width t0+block_width t0+block_width t0-block_width], ...
      [y0 y0 y0-block_height y0-block_height], block_color) ;

   set ([block1 block2], 'erasemode', 'xor') ;
   i = 1 ;
   for t = T'
      y = Y(i,1) ;
      set(block1,'xdata', -3/2*block_width+[t0-block_width t0+block_width t0+block_width t0-block_width], ...
         'ydata', [y y y-block_height y-block_height]) ;
		set(spring1, 'xdata', -3/2*block_width+t0+jagged, ...
         'ydata', linspace(maxy,y,spring_numjags)) ;
      set(block2,'xdata', [t-block_width t+block_width t+block_width t-block_width], ...
         'ydata', [y y y-block_height y-block_height]) ;
		set(spring2, 'xdata', t+jagged, ...
         'ydata', linspace(maxy,y,spring_numjags)) ;
      
   %   set(vel,'xdata',t+[block_width*4 block_width*4],...
   %      'ydata',[0 Y(i,2)]) ; 
   
   	drawnow
      for j=1:delay
      end
      
      i = i + 1 ;
      if abs(PointerLoc-get(0,'PointerLocation')) * [1 1]' > 20
         break 
      end
   end
   delete([block1 spring1 block2 spring2]) 
   set(solution, 'color', 'b') ;
   
case 'setu0',
   p = get(gca,'currentpoint') ;
   u0 = p(1,2) ;
   MassOnASpring update
   
case 'mslider',
   set(h.m,'string',num2str(get(h.mslider,'value'),'%2.1f')) 
   MassOnASpring update
   
case 'cslider',
   set(h.c,'string',num2str(get(h.cslider,'value'),'%2.2f')) 
   MassOnASpring update 
   
case 'kslider',
   set(h.k,'string',num2str(get(h.kslider,'value'),'%2.1f')) 
   MassOnASpring update
   
case 'stoptime',
   maxt = get(h.stoptimeslider,'value') ;
   set(h.stoptime,'string',num2str(maxt,'%2.1f')) ;
   MassOnASpring update
  
case 'help',
   helpwin MassOnASpring
   
case 'quit',
   delete (findobj ('tag', 'MassOnASpring')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
end



function h = CreateMassOnASpringWindow()
col = [.6 .8 1] ;

h0 = figure('Color', 'w', ...
   'MenuBar','none', ...
   'Name','MASS ON A SPRING', ...
   'NumberTitle','off', ...
   'Position',[15   20   620   410], ...
   'Resize','off', ...
   'Tag','MassOnASpring', ...
   'windowbuttondownfcn', 'MassOnASpring setu0') ;

uicontrol (h0, 'style', 'frame', ...
   'position', [0 0 630 160], ...
   'backgroundcolor', col) ;


% axes
h.ax = axes('Parent',h0, ...
   'Units','pixels', ...
   'Position',[40 165 560 230]);

uicontrol (h0, 'style', 'text', ...
   'position', [0 390 620 20], ...
   'string','mu''''(t) + cu''(t) + ku(t) = F(t)', ...
   'backgroundcolor', 'w') ;


h.startheight = uicontrol(h0,'style','text',...
   'position',[20 135 340 20],...
   'string', '', ...
   'backgroundcolor', col, ...
   'horizontalalignment', 'left');

% m: [ ]
uicontrol(h0,'backgroundcolor',col,...
   'horizontalalignment','left',...
   'style','text',...
   'string','m: ',...
   'position',[20 105 50 20]) ;
h.m = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[50 105 40 20], ...
   'String','1', ...
   'Style','edit', ...
   'Callback','MassOnASpring(''update'');');
h.mslider = uicontrol(h0,...
   'style','slider',...
   'position', [100 105 150 20], ...
   'min',1,...
   'max',50,...
   'value',1,...
   'callback','MassOnASpring(''mslider'');') ;


% c: [ ]
uicontrol(h0,'backgroundcolor',col,...
   'style','text',...
   'horizontalalignment','left',...
   'string','c: ',...
   'position',[20 75 50 20]) ;
h.c = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Position',[50 75 40 20], ...
   'String','.1', ...
   'Style','edit', ...
   'Callback','MassOnASpring(''update'');');
h.cslider = uicontrol(h0,...
   'style','slider',...
   'position', [100 75 150 20], ...
   'min',0,...
   'max',2,...
   'value',.1,...
   'callback','MassOnASpring(''cslider'');') ;


% k:  [   ]
uicontrol(h0,'backgroundcolor',col,...
   'style','text',...
   'string','k: ',...
   'position',[20 45 50 20], ...
   'horizontalalignment', 'left') ;
h.k = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Callback','MassOnASpring (''update'')', ...
   'Position',[50 45 40 20], ...
   'String','.5', ...
   'Style','edit');
h.kslider = uicontrol(h0,...
   'style','slider',...
   'position', [100 45 150 20], ...
   'min',0,...
   'max',10,...
   'value',.5,...
   'callback','MassOnASpring(''kslider'');') ;

% F(t): [   ]
uicontrol(h0,'backgroundcolor',col,...
   'style','text',...
   'string','F(t): ',...
   'position',[20 15 50 20], ...
   'horizontalalignment', 'left') ;
h.F = uicontrol('Parent',h0, ...
   'BackgroundColor','w', ...
   'Callback','MassOnASpring (''update'')', ...
   'HorizontalAlignment','left', ...
   'Position',[50 15 200 20], ...
   'String','0', ...
   'Style','edit');

% maxt:  [   ]
%uicontrol(h0,'backgroundcolor',col,...
%   'style','text',...
%   'string','stop time: ',...
%   'position',[260 45 50 20], ...
%   'horizontalalignment', 'left') ;
%h.stoptime = uicontrol('Parent',h0, ...
%   'BackgroundColor','w', ...
%   'Callback','MassOnASpring (''stoptime'')', ...
%   'Position',[300 45 40 20], ...
%   'String','10', ...
%   'Style','edit');
%h.stoptimeslider = uicontrol(h0,...
%   'style','slider',...
%   'position', [350 45 150 20], ...
%   'min',1,...
%   'max',100,...
%   'value',10,...
%   'callback','MassOnASpring(''stoptime'');') ;




% [Help] [Quit]
uicontrol('Parent',h0, ...
   'Callback','MassOnASpring (''animate'');', ...
   'Position',[390 100 110 30], ...
   'String','Animate');
uicontrol('Parent',h0, ...
   'Callback','MassOnASpring (''help'');', ...
   'Position',[390 65 110 30], ...
   'String','Help');
uicontrol('Parent',h0, ...
   'Callback','MassOnASpring (''quit'');', ...
   'Position',[390 30 110 30], ...
   'String','Quit');


