function result = paracurv(cmd, arg)
% PARAMETERIZED CURVES
%
% This tool is for graphing and analyzing parameterized curves in two
% dimensions.  Input the coordinate functions in the editable text boxes.
% The coordinates must be expressed as functions of the parameter t.
% 
% When you select the MARKER button, the cursor becomes a crosshairs.
% Position it over a point on the axis on the bottom of the main screen
% and click to select a value of the parameter t.  This value is marked on
% the t-axis by a green dot and the point [x(t), y(t)] is marked on the
% large coordinate plane by a red dot.  Marking may be done repeatedly.
% To clear all markers, select to CLEAR button.
% 
% You can "zoom in" on the t-axis or the coordinate plane:  Position the
% cursor over a point, and click.  This zooms you toward the point upon
% which you clicked, doubling the apparent length of any line segment. You
% may zoom in repeatedly.  Press SHIFT and click to zoom out.  Double click
% to return to the original graph or axis.
%
% 2.1/19-July-1998

% This is very much like polarcurves, so be sure to make changes to both
% in parallel.
%
% BUGS:
% - Green dots should be plotted on parameter axis.
%
% REQUIRES: cleanup.m
%
% HISTORY:
% - Original by Alex Gottlieb (Summer 1996)
% - Revised by Tom Insel (February 1998)
%   Revised for MATLAB 5, etc. (summer 1998)

persistent x_input y_input low_t high_t domain_axis range_axes

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	paracurv('regraph');

	control_figures = findobj('tag', 'paracurv_main');
	if length(control_figures) == 0
		control_figures = paracurv('makecontrol');
	end

	figure(control_figures(1));
	
elseif strcmp(cmd, 'makemain')

	control_figure = figure(...
		'position', [20 20 655 450],...
		'name', 'Parametrized Curves',...
		'numbertitle', 'off',...
		'tag', 'paracurv_main',...
		'resize', 'off',...
		'color', math_bkgnd,...
		'renderer', 'zbuffer',...
		'menubar', 'figure');
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 285 180 1]);
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'MARKERS',...
		'horizontalalignment', 'left',...
		'position', [10 255 60 20],...
		'backgroundcolor', math_bkgnd);		
		
	uicontrol(control_figure,...
		'string', 'Add',...
		'position', [30 220 65 25],...
		'callback', 'paracurv(''marker'');');
		
	uicontrol(control_figure,...
		'string', 'Clear',...
		'position', [100 220 65 25],...
		'callback', 'paracurv(''regraph'');');
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 210 180 1]);
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'THE CURVE',...
		'horizontalalignment', 'left',...
		'position', [10 180 60 20],...
		'backgroundcolor', math_bkgnd);
		
	% x(t), y(t)

	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'x(t)=',...
		'horizontalalignment', 'right',...
		'position', [10 155 60 20],...
		'backgroundcolor', math_bkgnd);

	x_input = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'sin(3*t)',...
		'position', [80 155 110 20],...
		'callback', 'paracurv(''regraph'');',...
		'backgroundcolor', math_field);
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'y(t)=',...
		'horizontalalignment', 'right',...
		'position', [10 130 60 20],...
		'backgroundcolor', math_bkgnd);
				
	y_input = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'sin(2*t)',...
		'position', [80 130 110 20],...
		'callback', 'paracurv(''regraph'');',...
		'backgroundcolor', math_field);
	
	% range of t values
		   	
	low_t = uicontrol(control_figure,...
		'style','edit',...
		'string', '0',...
	   'position', [20 105 60 20],...
		'callback', 'paracurv(''regraph'');',...
	   'backgroundcolor', math_field);

	if strcmp(computer,'MAC2')
		domain_string = ' t ';
	else
		domain_string = '<= t <=';
	end
	   
	uicontrol(control_figure,...
		'style', 'text',...
		'string', domain_string,...
		'position', [80 105 50 20],...
		'backgroundcolor', math_bkgnd);
   
	high_t = uicontrol(control_figure,...
		'style','edit',...
		'string', '2*pi',...
	   'position',[130 105 60 20],...
		'callback', 'paracurv(''regraph'');',...
	   'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 95 180 1]);
	   
	uicontrol(control_figure,...
		'string', 'Help',...
		'position', [10 50 180 35],...
		'callback', 'paracurv(''help'');');

	uicontrol(control_figure,...
		'string', 'Quit',...
		'position', [10 10 180 35],...
		'callback', 'paracurv(''end'');'); 

	% axes

	domain_axis = axes(...
		'position', [.4 .08 .55 .05],...
		'YTick', []);
	xlabel('the parameter t')
	zoom on
	 
	range_axes = axes('position', [.4 .18 .55 .8]);
	xlabel('x(t), y(t)')
	grid on
	zoom on 
		
	result = control_figure;
	
elseif strcmp(cmd, 'regraph')

	control_figures = findobj('tag', 'paracurv_main');
	if length(control_figures) == 0
		control_figures = paracurv('makemain');
	end
	
	try
		t0 = str2num(cleanup(get(low_t,'string')));
		t1 = str2num(cleanup(get(high_t,'string')));
	catch
		errmesg('t-domain', 'num')
		return
	end
	
	axes(domain_axis)
	axis([t0 t1 -eps eps])
	cla % to erase markers
	
	axes(range_axes)
	
	t = linspace(t0,t1,1000);
	
	try
		x = eval(vectorize(cleanup(get(x_input, 'string')))) + 0*t;
	catch
		errmesg('x(t)', 'num')
		return
	end

	try
		y = eval(vectorize(cleanup(get(y_input, 'string')))) + 0*t;
	catch
		errmesg('y(t)', 'num')
		return
	end
	
	plot(x,y)
		
elseif strcmp(cmd, 'marker')

	control_figures = findobj('tag', 'paracurv_main');
	if length(control_figures) == 0
		control_figures = paracurv('makemain');
	end
	
	axes(domain_axis)
	[t dummy] = ginput(1);
	
	if (gca == domain_axis)
		hold on
		plot(t, 0, 'g.', 'markersize', 20)
		hold off
		
		try
			c1 = eval(vectorize(cleanup(get(x_input, 'string'))));
		catch
			errmesg('x(t)', 'num')
			return
		end
		
		try
			c2 = eval(vectorize(cleanup(get(y_input, 'string'))));
		catch
			errmesg('y(t)', 'num')
			return
		end
		
		axes(range_axes)
		hold on
		plot(c1, c2, 'r.', 'markersize', 20)
		hold off
	else
		warndlg('You must click on the domain axis to add a marker.','Warning');
	end

elseif strcmp(cmd, 'help')

	helpwin paracurv
	
elseif strcmp(cmd, 'end')

	w = findobj('tag', 'paracurv_main');	
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end
	
end
