function result = polarcurves(cmd, arg)
% POLARCURVES
%
% This tool is for graphing and analyzing curves in the polar plane.
% Input the coordinate function r(t) in the editable text box.
%
% When you select the MARKER button, the cursor becomes a crosshairs.
% Position it over a point on the axis on the bottom of the main screen
% and click to select a value of the parameter t.  This value is marked
% on the t-axis by a green dot and the point with radius r(t) and angle
% t is marked on the large coordinate plane by a red circle.  Marking
% may be done repeatedly.  To clear all markers, select the CLEAR button.
%
% You can "zoom in" on the t-axis or the coordinate plane:  Position the
% cursor over a point, and click.  Press SHIFT and click to zoom out.
%
% 2.0/16-July-1998

% HISTORY:
% 1.0 -- original by Alex Gottlieb (summer 1996)
% 2.0 -- completed rewritten by Tom Insel spring/summer 1998.
%
% this is very much like paracurv, so be sure to make changes to both
% in parallel.

persistent  r_input low_t high_t domain_axis range_axes

math_bkgnd = [0.6 0.8 1.0];
math_field = [1 1 1];

if ~nargin, cmd = 'start'; end

if strcmp(cmd, 'start')

	polarcurves('regraph');

	control_figures = findobj('tag', 'polarcurves_main');
	if length(control_figures) == 0
		control_figures = polarcurves('makecontrol');
	end

	figure(control_figures(1));
	
elseif strcmp(cmd, 'makemain')

	control_figure = figure(...
		'position', [20 20 655 450],...
		'name', 'Polar Curves',...
		'numbertitle', 'off',...
		'tag', 'polarcurves_main',...
		'resize', 'off',...
		'renderer', 'zbuffer',...
		'color', math_bkgnd,...
		'menubar', 'figure');
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 250 180 1]);	
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'MARKERS',...
		'horizontalalignment', 'left',...
		'position', [10 220 60 20],...
		'backgroundcolor', math_bkgnd);		
		
	uicontrol(control_figure,...
		'string', 'Add',...
		'position', [30 195 65 25],...
		'callback', 'polarcurves(''marker'');');
		
	uicontrol(control_figure,...
		'string', 'Clear',...
		'position', [100 195 65 25],...
		'callback', 'polarcurves(''regraph'');');
		
	uicontrol(control_figure,...
		'style', 'frame',...
		'backgroundcolor', 'k',...
		'position', [10 185 180 1]);	
		
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'THE CURVE',...
		'horizontalalignment', 'left',...
		'position', [10 155 60 20],...
		'backgroundcolor', math_bkgnd);
		
	% x(t), y(t)

	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'r(t)=',...
		'horizontalalignment', 'right',...
		'position', [10 130 40 20],...
		'backgroundcolor', math_bkgnd);

	r_input = uicontrol(control_figure,...
		'style', 'edit',...
		'string', 'cos(2*t)',...
		'position', [60 130 130 20],...
		'callback', 'polarcurves(''regraph'');',...
		'backgroundcolor', math_field);
			
	% range of t values
	
	uicontrol(control_figure,...
		'style', 'text',...
		'string', 'range of t values',...
		'horizontalalignment', 'right',...
		'position', [10 105 90 20],...
		'backgroundcolor', math_bkgnd);
	   	
	low_t = uicontrol(control_figure,...
		'style','edit',...
		'string', '0',...
	   'position', [110 105 40 20],...
		'callback', 'polarcurves(''regraph'');',...
	   'backgroundcolor', math_field);
	   
	high_t = uicontrol(control_figure,...
		'style','edit',...
		'string', '2*pi',...
	   'position',[150 105 40 20],...
		'callback', 'polarcurves(''regraph'');',...
	   'backgroundcolor', math_field);
	   
	uicontrol(control_figure,...
		'style', 'frame',...
		'position', [10 95 180 1]);
	   
	uicontrol(control_figure,...
		'string', 'Help',...
		'position', [10 50 180 35],...
		'callback', 'helpwin polarcurves');

	uicontrol(control_figure,...
		'string', 'Quit',...
		'position', [10 10 180 35],...
		'callback', 'polarcurves(''end'');'); 
	   
	% axes
	
	domain_axis = axes(...
		'position', [.4 .08 .55 .05],...
		'YTick', []);
	xlabel('the parameter t')
	zoom on
	 
	range_axes = axes(...
		'position', [.4 .18 .55 .8],...
		'visible', 'off'); % invisible, so that we never see the square axes.
	zoom on 
		
	result = control_figure;
	
elseif strcmp(cmd, 'regraph')

	control_figures = findobj('tag', 'polarcurves_main');
	if length(control_figures) == 0
		control_figures = polarcurves('makemain');
	end
	
	try
		t0 = str2num(cleanup(get(low_t,'string')));
		t1 = str2num(cleanup(get(high_t, 'string')));
	catch
		errmesg('t-domain', 'num')
		return
	end
		
	axes(domain_axis)
	axis([t0 t1 -eps eps])
	cla % to erase markers
	
	axes(range_axes)
			
	t = linspace(t0,t1,1000);
	
	try
		r = eval(vectorize(cleanup(get(r_input, 'string')))) + 0*t;
	catch
		errmesg('r(t)','fun')
		return
	end
	
	polar(t,r)
			
elseif strcmp(cmd, 'marker')

	axes(domain_axis)
	[t dummy] = ginput(1);

	if (gca == domain_axis)
		hold on
		plot(t, 0, 'g.', 'markersize', 20)
		hold off
		
		try
			r = eval(vectorize(cleanup(get(r_input, 'string'))));
		catch
			errmesg('r(t)','fun')
			return
		end
		
		axes(range_axes)
		hold on
		polar(t, r, 'ro')
		hold off
	else
		warndlg('You must click on the domain axis to add a marker.','Warning');
	end

elseif strcmp(cmd, 'end')

	w = findobj('tag', 'polarcurves_main');	
	for i = 1:length(w)
		delete(w(i))
	end

	% if it's open, return to the main menu

	w = findobj('tag', 'math_main_menu');
	if length(w)
		figure(w(1));
	end
	
end
