% SEQUENCES AND SERIES
%
% This program plots the terms of a sequence or the partial sums
% of a series.  It also computes the limit of a sequence or the
% sum of a series.  
%
% INSTRUCTIONS
% 1. Select either Series or Sequence.
% 2. Enter the nth term in the a_n box. 
% 3. Use the slider bar to adjust the number of terms to be plotted, 
%    or click on 'Next Term' to plot just one more term. 
%
% There are three display areas.  The first shows the limit
% (resp., sum) of the sequence (resp., series).  The next
% shows the terms of the sequence (or partial sums) graphed
% as a function of n.  The third shows the same information
% but graphed on a number line with each point labeled.  
% On each graph there may be a blue line which indicates the
% limit of the sequence (or the sum of the series).
%
% WARNING! WARNING! These limits and sums are computed 
% symbolically using the Maple engine which sometimes 
% gets things completely wrong.  Try summing (-1)^n.
%
% William Stein, (was@math.berkeley.edu), July, 1998.


function SequencesAndSeries(action)

persistent h

if nargin == 0
   action = 'Init' ;
end

switch (action)
case 'Init',
   h_fig = findobj ('tag', 'SequencesAndSeries') ;
   if h_fig
      figure (h_fig)
   else
      h = CreateSequencesAndSeriesWindow ;
   	SequencesAndSeries('Update') ; 
   end
   
case 'Update',
  %try
      series = get (h.seriesbutton, 'Value') ;
	  an 	= cleanup (char(get (h.an, 'String'))) ;
      nmax = str2num(cleanup (get (h.nmax, 'String'))) ;
      
      % first compute (or reload) the limit 
      % of the sequence (or sum of the series), 
      % if it exists.
	  if havesymbolic
      	n=sym('n') ;
      end
		
      % the user data of the limit text box tells whether or
      % not the limit has been computed:
      % 0 = limit not computed
      % 1 = limit computed and valid
      % 2 = can not compute the limit
      computed = get (h.limit, 'Userdata') ;
      if ~havesymbolic 
		  computed = 2;
	  end
	  
      if computed == 0    % we haven't already computed the limit
     
	     % try to compute it symbolically
         if series 
            lim = symsum ( simple ( sym(an) ), 1, inf ) ;
         else
            lim = limit ( simple ( sym(an) ), n, inf ) ;
         end
     
         limstr = char (lim) ; 
         
         limexists = 1 ;
      
         % check if the symbolic computation worked.
         % if not, try a "numerical method"
         if ~ isempty(findstr(limstr, '..')) | ...
               strcmp(limstr,'NaN') | ~isempty(findstr(limstr, 'limit'))
            if series 
               
               % just take the sum of first 500 terms.  really approximate.
				   if (limit ( simple ( sym(an) ), n, inf ) == 0) 
	               n = 1:500 ;
   	            lim = sum (eval(vectorize(an))) ;
      	         set (h.limit, 'String', num2str(lim)) ;
	               set (h.limit, 'Userdata', 1) ;
               else
                  set (h.limit, 'String', '?') ;
                  lim = inf ;
                  set (h.limit, 'Userdata', 2) ;
               end 
                            
            else
               % "algorithm" to numerically approximate a limit
               n = 500 ;
               lim = eval(vectorize(an)) ;
               n = 501 ;
               lim2 = eval(vectorize(an)) ;
               n = 502 ;
               lim3 = eval(vectorize(an)) ;
               
               n = 400:500 ;
               vals = eval(vectorize(an)) ;      
               tol = (max(vals) - min(vals)) / 10000 ;
               if abs(lim-lim2)>tol | abs(lim-lim3)>tol
                  set (h.limit, 'String', '?') ;
                  set (h.limit, 'Userdata', 2) ;
                  limexists = 0;
               else
                  set (h.limit, 'String', num2str(lim)) ;
                  set (h.limit, 'Userdata', 1) ;
               end
             end
         else
            
            set (h.limit, 'String', char(lim)) ;
            set (h.limit, 'Userdata', 1) ;
         	lim = eval(lim) ;
         end
         
      else
         
         if computed == 1
            
            lim = eval (get (h.limit, 'String')) ;
            limexists = 1;
            
         else
            
            limexists = 0 ;
   		end
	end
	
         
      % next graph the terms of the sequence as points on the real line
      if series
         anvals = zeros (1,nmax) ;
         n = 1 ;
         anvals (1) = eval (an) ;
         for i = 2:nmax 
            n = i ;
            anvals (i) = anvals (i-1) + eval (an) ;
         end

      else
         n = 1:nmax ;
         anvals = eval (vectorize (an)) + zeros(1,nmax);
      end
      
      axes (h.ax1) 
      hold off
      plot (anvals, zeros (1,nmax), '*r') ;
      hold on
      % add labels to the first few points (until they start getting bunched up).
      for m = 1:min(nmax,9)
         if series
	         text (anvals(m), .4, ['s_' num2str(m)]) 
			else	
            text (anvals(m), .4, ['a_' num2str(m)]) 
			end            
      end
      xlim = get (gca, 'xlim') ;
      set (gca, 'xlim', xlim + [-1,1] * (xlim(2)-xlim(1))/6) 
      if limexists 
         ylim = get (gca, 'ylim') ;
         plot ([1,1]*lim, ylim, '-b') ;
      end
      
      % finally graph the terms of the sequence as a function of n.
      axes (h.ax2) 
      hold off 
      plot (1:nmax,anvals, '*r') ;
      hold on ;
      xlim = get (gca,'xlim') ;
      new_xlim = xlim + [-1, 1] * (xlim(2) - xlim(1)) / 6 ;
      set (gca, 'xlim', new_xlim) 
      ylim = get (gca,'ylim') ;
      new_ylim = ylim + [-1, 1] * (ylim(2) - ylim(1)) / 8 ;
      set (gca, 'ylim', new_ylim) 
      if limexists 
         plot (new_xlim,[1,1]*lim, '-b') ;
      end
%   catch
%      errmesg('nth term of sequence', 'fun');
%      error(lasterr);
%   end

case 'SequenceButton',
   set (h.sequencebutton, 'value', 1) 
   set (h.seriesbutton, 'value', 0) 
   set (findobj ('tag','SeriesText'), 'visible', 'off') 
   set (findobj ('tag','SequenceText'), 'visible', 'on') 
   SequencesAndSeries 'NewFunction'
   
case 'SeriesButton',
   set (h.sequencebutton, 'value', 0) 
   set (h.seriesbutton, 'value', 1) 
   set (findobj ('tag','SequenceText'), 'visible', 'off') 
   set (findobj ('tag','SeriesText'), 'visible', 'on') 
   SequencesAndSeries 'NewFunction'

case 'NewFunction',
   set (h.limit, 'Userdata', 0) ;
	SequencesAndSeries ('Update') ;   
   
case 'Slider', 
   val = num2str (round (get (gcbo, 'Value'))) ;
   set ( h.nmax, 'String', num2str(val) ) ;
   SequencesAndSeries ('Update') ;
   
case 'ChangeMaxN',
   val = str2num (cleanup (get (h.nmax, 'String'))) ;
   set ( h.nmaxslider, 'Value', min(100,val) ) ;
	SequencesAndSeries ('Update') ;
   
case 'AddOneMore',
   n = str2num (cleanup (get (h.nmax, 'String'))) + 1;
   set (h.nmax, 'String', num2str(n)) ;
   set (h.nmaxslider, 'Value', n) ;
   an = cleanup (get (h.an, 'String') );
   SequencesAndSeries('Update') ;
   
case 'Help',
   helpwin sequencesandseries
   
case 'Quit', 
   delete (findobj ('tag', 'SequencesAndSeries')) 
   w = findobj ('tag', 'math_main_menu') ;
   if w
      figure(w(1))
   end
end


function h = CreateSequencesAndSeriesWindow ()
col = [.6 .8 1] ;
edcol = 'w' ;

h0 = figure('Color', col, ...
	'MenuBar','none', ...
	'Name','SEQUENCES AND SERIES', ...
	'NumberTitle','off', ...
	'Position',[10 10 620 410], ...
	'Resize','on', ...
   'Tag','SequencesAndSeries');




% a_n = [ ]
uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Position',[20 10 15 20], ...
	'String','a', ...
	'Style','text');
uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[30 3 15 20], ...
	'String','n', ...
	'Style','text');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'Position',[40 10 15 20], ...
	'String','=', ...
   'Style','text');
h.an = uicontrol('Parent',h0, ...
	'BackgroundColor',edcol, ...
	'Callback','SequencesAndSeries (''NewFunction'') ;', ...
	'HorizontalAlignment','left', ...
	'Position',[60 10 250 20], ...
	'String','1/n^2', ...
	'Style','edit');



% Number of terms to plot []  [<--[]---->]
uicontrol('Parent',h0, ...
   'BackgroundColor', col, ...
   'horizontalalignment', 'left', ...
	'Position',[320 45 140 20], ...
	'String','Number of terms:', ...
   'Style','text');
h.nmax = uicontrol('Parent',h0, ...
	'BackgroundColor',edcol, ...
	'Callback','SequencesAndSeries (''ChangeMaxN'')', ...
	'Position',[427 50 45 20], ...
	'String','8', ...
	'Style','edit');
h.nmaxslider = uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Callback','SequencesAndSeries (''Slider'')', ...
	'Min',1, 'Max',100, ...
	'Position',[480 50 130 20], ...
	'Style','slider', ...
   'Value',10);










% [Next Point]  [Help]  [Quit] 
uicontrol('Parent',h0, ...
	'Callback','SequencesAndSeries(''AddOneMore'')', ...
	'Position',[320 10 90 30], ...
	'String','Next Point');
uicontrol('Parent',h0, ...
	'Callback','SequencesAndSeries(''Help'')', ...
	'Position',[420 10 90 30], ...
	'String','Help');
uicontrol('Parent',h0, ...
	'Callback','sequencesandseries(''Quit'')', ...
	'Position',[520 10 90 30], ...
   'String','Quit');





% [] sequence   [] series
h.seriesbutton = uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Callback','SequencesAndSeries(''SeriesButton'')', ...
   'Position',[20 45 50 20], ...
	'String','Series', ...
	'Style','radiobutton', ...
	'Tag','Series');
h.sequencebutton = uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'Callback','SequencesAndSeries(''SequenceButton'')', ...
	'Position',[80 45 70 20], ...
	'String','Sequence', ...
	'Style','radiobutton', ...
	'Value',1);





% lim_{n-->inf} a_n = [ ]
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
   'Position',[40 380 25 15], ...
   'horizontalalignment', 'left',...
	'String','lim', ...
   'Style','text', ...
   'Tag', 'SequenceText');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'HorizontalAlignment','left', ...
	'Position',[40 365 40 15], ...
	'String','n-->oo', ...
   'Style','text', ...
   'Tag', 'SequenceText');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
   'Position',[80 380 15 15], ...
   'horizontalalignment', 'left', ....
	'String','a', ...
   'Style','text', ...
   'Tag', 'SequenceText') ;
uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[88 372 15 15], ...
	'String','n', ...
   'Style','text',...
   'Tag', 'SequenceText');
uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
   'Position',[100 380 15 15], ...
	'HorizontalAlignment','left', ...
	'String','=', ...
   'Style','text', ...
   'Tag', 'SequenceText');
h.limit = uicontrol('Parent',h0, ...
	'BackgroundColor', col, ...
	'HorizontalAlignment','left', ...
	'Position',[160 380 200 15], ...
	'String','0', ...
	'Style','text', ...
	'UserData',1);





% a_1 + a_2 + ... 
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'Position',[40 380 35 15], ...
	'HorizontalAlignment','left', ...
	'String','a', ...
	'Style','text', ...
	'Tag','SeriesText', ...
	'Visible','off');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'HorizontalAlignment','left', ...
	'Position',[48 372 15 15], ...
	'String','1', ...
	'Style','text', ...
	'Tag','SeriesText', ...
	'Visible','off');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
	'HorizontalAlignment','left', ...
	'Position',[55 380 35 15], ...
	'String','+ a', ...
	'Style','text', ...
	'Tag','SeriesText', ...
	'Visible','off');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
   'Position',[72 372 15 15], ...
 	'HorizontalAlignment','left', ...
	'String','2', ...
	'Style','text', ...
	'Tag','SeriesText', ...
	'Visible','off');
uicontrol('Parent',h0, ...
	'BackgroundColor',col, ...
   'Position',[90 380 50 15], ...
 	'HorizontalAlignment','left', ...
	'String','+ . . .  = ', ...
	'Style','text', ...
	'Tag','SeriesText', ...
	'Visible','off');



% ax1 and ax2.
h.ax1 = axes('Parent',h0, ...
	'Units','pixels', ...
	'Box','on', ...
	'Position',[40 95 570 130]);

h.ax2 = axes('Parent',h0, ...
	'Units','pixels', ...
	'Box','on', ...
	'Position',[40 260 570 90]);

