function TaylorSeries(cmd, param)
% TAYLOR SERIES 
%
% This module plots finite Taylor series approximations.
%
% 1. Enter f(x) and the range over which to plot.
% 2. Set n equal to the degree of the approximating Taylor 
%    polynomial.  
% 3. Set a equal to the point about which to expand.
%    Note that a can also be set by moving the mouse.
%
% AUTHOR: William Stein, March, 1998.

% REQUIRES: cleanup.m, errmesg.m
%
% EXAMPLE: Type RSum; to run this module.
%          No arguments are needed.
%
%         Based on a program by Julie Mitchel, 1996.

xResolution = 400;

h_figs=findobj('Tag','tag_TaylorSeries');

if length(h_figs)>0 
   h_fig=h_figs(1);
   if nargin==0
      % When an instance of TaylorSeries is already running, the
      % command TaylorSeries just brings it to the foreground.
      figure(h_fig);
      return
   end
end

if nargin==0 
	if ~havesymbolic
		errordlg('This module requires the symbolic toolkit.','Taylor Series')
		return
	end

	cmd='Initialize';
end

if ~strcmp(cmd, 'Initialize')
   if length(h_figs)==0 
      % No non-initialization commands should be called if 
      % the application has not yet been created.
      disp('Error!: TaylorSeries has not yet been initialized.');
      return
   end
   
   % Retrieve handles and redefine the handle variables.
   h=get(h_fig,'Userdata'); 
   h_mark=h(1); h_n=h(2); h_a=h(3); h_f=h(4); h_fplot=h(5); 
   h_Tplot=h(6); h_xmin=h(7); h_xmax=h(8); 
   % ...
end

switch cmd
case 'Initialize',
   h_fig=figure('Position',[50 54 600 410],...
      'Tag','tag_TaylorSeries',...
      'NumberTitle','off',...
      'Menu', 'none',...
      'Name','TAYLOR SERIES',...
      'Resize','off');
   
   figure (h_fig) 
   
   % Create the objects: 
   % axes: f plot, T plot, marker point.
   % f(x)=[?],   [?]< x < [?]
   % Expand about: a=[ ]  [Animate] as a varies.
   % Degree of Taylor polynomial: n=[ ]   [Animate] as n varies between 1 and 50
   % [Help]
   
   % Create axes
   h_ax=axes('units','pixels',...
      'position',[40 165 520 220]);
   
   drawnow
   set(gcf,'WindowButtonMotionFcn','TaylorSeries(''MouseSelectA'');')
   
   
   % Create plots
   hold on
   h_fplot=plot([0],[0],'b');
   h_Tplot=plot([0],[0],'g');
   h_mark=plot([0],[0],'xr');
   set([h_fplot h_Tplot h_mark],'erasemode','xor') 
   hold off;
   
   % Create axes labels
   title('green line = f(a)+f''(a)(x-a)+...+f^{(n)}(a)(x-a)^n/n!')
   xlabel('x')
   ylabel('y')
   
   % Create control frames
   col=[0.7 0.9 1.0];
   uicontrol(h_fig,'style','frame',...
      'position', [10 10 580 120],...
      'background', col);
   uicontrol(h_fig,'style','frame',...
      'position', [10 79 580 51],...
      'background', col);
   uicontrol(h_fig,'style','frame',...
      'position', [10 10 450 70],...
      'background', col);
      
   % Create function input box: f(x)=[ ]
   uicontrol(h_fig,'style','text',...
      'position', [20 92 45 15],...
      'string', 'f(x) =',...
      'HorizontalAlignment', 'Left',...
      'background', col);
   h_f=uicontrol(h_fig,'style','edit',...
      'position', [70 95 260 20],...
      'background', 'w',...
      'HorizontalAlignment', 'Left',...
      'callback', 'TaylorSeries(''Update'',''nonempty'')',...
      'String','sin(x)');
   
   % Create xmin and xmax fields.
   h_xmin=uicontrol(h_fig,'style','edit',...
      'position',[405 95 40 20],...
      'background','w',...
      'Callback','TaylorSeries(''Update'',''nonempty'')',...
      'String','-4');
   uicontrol(h_fig,'style','text',...
      'position',[450 92 40 15],...
      'background',col,...
      'String','<  x  <');
   h_xmax=uicontrol(h_fig,'style','edit',...
      'position',[500 95 40 20],...
      'background','w',...
      'Callback','TaylorSeries(''Update'',''nonempty'')',...
      'String','4');
   
   % a 
   uicontrol(h_fig,'style','text',...
      'position',[20 50 100 15],...
      'background',col,...
      'HorizontalAlignment', 'Left',...
      'String','Expand about: a =');
   h_a = uicontrol(h_fig,'style','edit',...
      'position',[115 50 40 20],...
      'background','w',...
      'Callback','TaylorSeries(''Update'',''nonempty'')',...
      'String','0');
   uicontrol(h_fig,'String','<',...
      'position', [160 50 30 20],...
      'Callback','TaylorSeries(''Change'',''Decrease_a   '')');
   uicontrol(h_fig,'String','>',...
      'position', [190 50 30 20],...
      'Callback','TaylorSeries(''Change'',''Increase_a   '')');    
   % n
   uicontrol(h_fig,'style','text',...
      'position',[20 20 180 15],...
      'background',col,...
      'HorizontalAlignment', 'Left',...
      'String','Degree of Taylor approximation: n =');
   h_n = uicontrol(h_fig,'style','edit',...
      'position',[200 20 40 20],...
      'background','w',...
      'Callback','TaylorSeries(''Update'', ''nonempty'')',...
      'String','3');
   uicontrol(h_fig,'String','<',...
      'position', [245 20 30 20],...
      'Callback','TaylorSeries(''Change'',''Decrease_n   '')');
   uicontrol(h_fig,'String','>',...
      'position', [275 20 30 20],...
      'Callback','TaylorSeries(''Change'',''Increase_n   '')');   
   
   % Examples button
   uicontrol(h_fig,'String','Examples',...
      'position',[338 47 110 25],...
      'Callback','TaylorSeries(''Examples'')');
   
   
   % Help button
   uicontrol(h_fig,'String','Help',...
      'position',[470 47 110 25],...
      'Callback','TaylorSeries(''Help'')');
   
   % Quit button
   uicontrol(h_fig,'String','Quit',...
      'position',[470 17 110 25],...
      'Callback','TaylorSeries(''Quit'')');
   	   
   % Save handles in userdata.
   set(h_fig,'userdata',[h_mark h_n h_a h_f ...
         h_fplot h_Tplot h_xmin h_xmax]);
   
   % Draw the plot.
   TaylorSeries('Update','nonempty');   
   
case 'Update',   
   
   field=0; 
   s_xmin=cleanup(get(h_xmin,'String'));
   eval([s_xmin ';'], 'field=''Minimum x Value'';');
   s_xmax=cleanup(get(h_xmax,'String'));
   eval([s_xmax ';'], 'field=''Maximum x Value'';');
   s_n=cleanup(get(h_n,'String'));
   if (str2num(s_n)<1)
      s_n='2';
      set(h_n,'string',s_n)
   end
   
   eval([s_n ';'], 'field=''Degree of Taylor polynomial: n'';');
   s_a=cleanup(get(h_a,'String'));
   eval([s_a ';'], 'field=''Point to expand about: a'';');

   if field
      errmesg(field,'num'); error(lasterr);
   end      
   
   xmin=str2num(s_xmin);
   xmax=str2num(s_xmax);
   
   if xmin>xmax
      temp=xmin; xmin=xmax; xmax=temp;
      set(h_xmin,'String',num2str(xmin));
      set(h_xmax,'String',num2str(xmax));
   end
   
   if xmin==xmax
      xmin=xmax-.001;
      set(h_xmin,'String',num2str(xmin));
   end
   
   n=round(str2num(s_n));
   a=str2num(s_a);
   
   if n<=0 
      n=0; set(h_n,'String','0');
   elseif n>100  
      n=100; set(h_n,'String','100');
   end
   
   if a<xmin
      a=xmin;
      set(h_a,'String',num2str(a));
    elseif a>xmax
      a=xmax;
      set(h_a,'String',num2str(a));
   end
   
   x=linspace(xmin,xmax,xResolution);
   
   f=vectorize([cleanup(get(h_f,'String')) '+0*x']);
   field=0;
   eval([f ';'], 'field=''f(x)'';');
   if field
      errmesg(field,'fun'); error(lasterr);
   end   
   
   T = vectorize ( ComputeTaylorSeries ( cleanup (get (h_f,'String') ), n, a) ) ;
   
   
   if nargin > 1
	   y = eval (f) ;
   	set(gca,'XLim',[xmin xmax]);
	   if min(y)<max(y)
   	  height=max(y)-min(y);
	   	 set(gca,'YLim',[min(y)-height/6,max(y)+height/6]);
	   else
   		 set(gca,'YLim',[min(y)-.001 min(y)+.001]);
	   end
      set(h_fplot,'XData',x, 'YData',y);
   end
   
   
	set(gca,'YLimMode','Manual');
   if isempty(T)
      set(h_Tplot,'XData',x, 'YData', eval(f));
   else
      set(h_Tplot,'XData',x, 'YData', eval([T '+0*x']));
   end
      
   
   x=a; 
   set(h_mark,'XData',x, 'YData', eval(f));
   
   drawnow;
   
case 'Help',
   helpwin ('TaylorSeries.m') 
   
case 'Change',
   field=0; 
   s_xmin=cleanup(get(h_xmin,'String'));
   eval([s_xmin ';'], 'field=''Minimum x Value'';');
   s_xmax=cleanup(get(h_xmax,'String'));
   eval([s_xmax ';'], 'field=''Maximum x Value'';');
   s_n=cleanup(get(h_n,'String'));
   eval([s_n ';'], 'field=''Degree of Taylor polynomial: n'';');
   s_a=cleanup(get(h_a,'String'));
   eval([s_a ';'], 'field=''Point to expand about: a'';');

   if field
      errmesg(field,'num'); error(lasterr);
   end      
   
   xmin=str2num(s_xmin);
   xmax=str2num(s_xmax);
   n=round(str2num(s_n));
   a=str2num(s_a);
   step=abs((xmax-xmin)/20);
   
   if param == 'Increase_a   '
      a = a + step;   set(h_a,'String',num2str(a));
   elseif param == 'Decrease_a   '
      a = a - step;   set(h_a,'String',num2str(a));
   elseif param == 'Increase_n   '
      n = n+1;   set(h_n,'String',num2str(n));
   elseif param == 'Decrease_n   '
      n = n-1;   set(h_n,'String',num2str(n));
      
   elseif param == 'Increase_xmax'
      xmax = xmax + step;  set(h_xmax,'String',num2str(xmax));
   elseif param == 'Decrease_xmax'
      xmax = xmax - step;  set(h_xmax,'String',num2str(xmax));
   elseif param == 'Increase_xmin'
      xmin = xmin + step;  set(h_xmin,'String',num2str(xmin));
   elseif param == 'Decrease_xmin'
      xmin = xmin - step;  set(h_xmin,'String',num2str(xmin));
      
   end
   TaylorSeries('Update');
   
case 'Examples',
   e(1).button = 'sin(x)' ;
   e(1).text = 'The sine function.' ;
   % data is of the form: function, xmin, xmax
   e(1).data = {'sin(x)', '-3*pi', '3*pi'} ;
   e(2).button = 'cos(x)' ;
   e(2).text = 'The cosine function.' ;
   e(2).data = {'cos(x)', '-3*pi', '3*pi'} ;
   e(3).button = 'e^x' ;
   e(3).text = 'The exponential function.' ;
   e(3).data = {'e^x', '-1', '8'} ;
   e(4).button = 'e^(1/x)' ;
   e(4).text = 'A function with an essential singularity at 0.' ;
   e(4).data = {'e^(1/x)', '.1', '.5'} ;
 	i = examples('TAYLOR SERIES', e) ;
   if i
      e = e(i).data ;
      set (h_f, 'string', e{1}) ;
      set (h_xmin, 'string', e{2}) ;
      set (h_xmax, 'string', e{3}) ;
      set (h_a, 'string', e{2}) ;
      TaylorSeries('Update','nonempty') 
   end
   
case 'MouseSelectA',
 	loc = get(gcf, 'CurrentPoint') ;
  	r = get(gca,'Position') ;
   if (loc(1) > r(1)) & (loc(1) < r(1)+r(3)) & ...
         (loc(2) > r(2)) & (loc(2) < r(2)+r(4))   
      set(gcf,'pointer','cross');
      p=get(gca,'CurrentPoint');
	   set(h_a,'String',num2str(p(1,1)));
      TaylorSeries('Update'); 
   else
      set(gcf,'pointer','arrow')
   end
   
   
case 'Quit',
   delete (findobj ('tag', 'tag_TaylorSeries')) ;
   w=findobj ('tag', 'math_main_menu') ;
   if length(w)
      figure(w(1));
   end
end


function T = ComputeTaylorSeries (f, n, a)
	s = maple (['taylor(' char(f) ',x=' num2str(a) ',' num2str(n) ')']) ;
	k = findstr (s,'+O') ;
	T = s(8:(k-1)) ;


